"""Contains several tools and utils for WordPress Themes"""
import devops_toolset.filesystem.parsers as parsers
import devops_toolset.filesystem
import devops_toolset.project_types.node.npm as npm
import devops_toolset.filesystem.paths as paths
import devops_toolset.filesystem.tools
import devops_toolset.filesystem.zip
import json
import logging
import os
import pathlib
import devops_toolset.project_types.wordpress.wp_cli as wp_cli
import devops_toolset.project_types.wordpress.constants as wp_constants
import devops_toolset.project_types.wordpress.wptools as wptools
import re
import devops_toolset.tools.git as git_tools


from devops_toolset.core.app import App
from devops_toolset.core.CommandsCore import CommandsCore
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.devops_platforms import constants as devops_platforms_constants
from devops_toolset.devops_platforms.azuredevops.Literals import Literals as PlatformLiterals
from devops_toolset.project_types.wordpress.basic_structure_starter import BasicStructureStarter
from devops_toolset.project_types.wordpress.commands import Commands as WordpressCommands
from devops_toolset.project_types.wordpress.Literals import Literals as WordpressLiterals
from devops_toolset.tools import cli

app: App = App()
platform_specific_restapi = app.load_platform_specific("restapi")
literals = LiteralsCore([WordpressLiterals])
platform_literals = LiteralsCore([PlatformLiterals])
commands = CommandsCore([WordpressCommands])


def build_theme(themes_config: dict, theme_path: str, root_path: str):
    """ Builds a theme source into a packaged theme distribution using npm tasks

    Args:
        themes_config: Themes configuration.
        theme_path: Path to the theme in the WordPress repository.
        root_path: Path to the project root.
    """
    logging.info(literals.get("wp_looking_for_src_themes"))

    # Get configuration data and paths
    src_theme_list = list(filter(lambda elem: elem["source_type"] == "src", themes_config))

    if len(src_theme_list) == 0:
        # Src theme not present
        logging.info(literals.get("wp_no_src_themes"))
        return

    src_theme = src_theme_list[0]
    theme_path_obj = pathlib.Path(theme_path)
    theme_path_src = pathlib.Path.joinpath(theme_path_obj, src_theme["source"])
    theme_path_dist = pathlib.Path.joinpath(theme_path_src, "dist")
    theme_path_zip = pathlib.Path.joinpath(theme_path_obj, f"{src_theme['name']}.zip")

    if os.path.exists(theme_path_src):

        theme_slug = src_theme["name"]

        # Change to the the theme's source directory
        os.chdir(theme_path_src)

        # Run npm install from the package.json path
        npm.install()

        # Run npm run build to execute the task build with the required parameters
        cli.call_subprocess(commands.get("wp_theme_src_build").format(
            theme_slug=theme_slug,
            path=theme_path_dist
        ), log_before_out=[literals.get("wp_gulp_build_before").format(theme_slug=theme_slug)],
            log_after_out=[literals.get("wp_gulp_build_after").format(theme_slug=theme_slug)],
            log_after_err=[literals.get("wp_gulp_build_error").format(theme_slug=theme_slug)])

        # Zip dist
        devops_toolset.filesystem.zip.zip_directory(
            theme_path_dist.as_posix(), theme_path_zip.as_posix(), f"{theme_slug}/")

        # Replace project.xml version with the one in the package.json file
        package_json_path = str(pathlib.Path.joinpath(pathlib.Path(theme_path_src, "package.json")))
        project_xml_path = str(pathlib.Path.joinpath(pathlib.Path(root_path), "project.xml"))
        package_json = parsers.parse_json_file(package_json_path)
        devops_toolset.filesystem.tools.update_xml_file_entity_text(
            "./version", package_json["version"], project_xml_path)

    else:
        logging.error(literals.get("wp_file_not_found").format(file=theme_path_src))


def check_theme_configuration(theme: dict) -> bool:
    """Checks that the themes configuration is correct.

    Args:
        theme: Themes configuration.

    Returns:
        True if the configuration is correct, False if incorrect.
    """

    # Check if feed-based themes contain feed information
    if theme["source_type"] == "feed" and "feed" not in theme:
        logging.error(literals.get("wp_theme_feed_no_info").format(theme=theme["name"]))
        logging.warning(literals.get("wp_themes_install_manually"))
        return False

    return True


def check_themes_configuration(themes: dict) -> bool:
    """Checks that the themes configuration is correct.

    Args:
        themes: Themes configuration.

    Returns:
        True if the configuration is correct, False if incorrect.
    """

    # Check if the number of themes is correct
    themes_number = len(themes)
    if themes_number == 0 or themes_number > 2:
        logging.error(literals.get("wp_config_file_bad_themes_number").format(number=themes_number))
        logging.warning(literals.get("wp_themes_install_manually"))
        return False

    # Check if the number of themes to be activated is more or less than one
    themes_to_activate: int = 0
    for theme in themes:
        if theme["activate"]:
            themes_to_activate += 1
    if themes_to_activate == 0 or themes_to_activate > 1:
        logging.error(literals.get("wp_config_file_only_one_activated_theme").format(number=themes_to_activate))
        logging.warning(literals.get("wp_themes_install_manually"))
        return False

    return True


def create_development_theme(theme_configuration: dict, root_path: str, constants: dict):
    """ Creates the structure of a development theme.

    Args:
        theme_configuration: The themes configuration node
        root_path: The desired destination path of the theme
        constants: WordPress constants.
    """

    destination_path = get_themes_path_from_root_path(root_path, constants)

    # Extract theme name from the themes configuration dict
    src_theme = list(filter(lambda t: t["source_type"] == "src", theme_configuration))
    if src_theme.__len__() == 1:
        src_theme = src_theme[0]
        theme_slug = src_theme["source"]

        # Check if a structure file is available (should be named as [theme-slug]-wordpress-theme-structure
        structure_file_name = f'{theme_slug}-wordpress-theme-structure.json'
        structure_file_path = str(pathlib.Path.joinpath(pathlib.Path(root_path), structure_file_name))

        # Create the structure based on the theme_name
        start_basic_theme_structure(destination_path, theme_slug, structure_file_path)

        # Replace necessary theme files with the theme name.
        set_theme_metadata(root_path, src_theme)
    else:
        logging.warning(literals.get("wp_src_theme_not_found"))


def download_wordpress_theme(theme_config: dict, destination_path: str, **kwargs):
    """Downloads a WordPress theme from a feed or a URL.

    NOTE: The URL must download a zip file that contains the theme. If the ZIP
        contains a non-standard inner structure, the calling process will
        produce side-effects.

    Args:
        theme_config: Theme configuration.
        destination_path: Path where the theme will be downloaded.
        kwargs: Platform-specific arguments
    """
    source_type: str = theme_config["source_type"]

    if source_type == "feed":
        feed_config = theme_config["feed"]
        matches = re.search(r"https:\/\/dev.azure.com\/(\w+)\/", feed_config["organization_url"])
        organization = matches.group(1)
        if "azdevops_user" in kwargs and "azdevops_token" in kwargs:
            platform_specific_restapi.get_last_artifact(organization, feed_config["name"], feed_config["package"],
                                                        destination_path, kwargs["azdevops_user"],
                                                        kwargs["azdevops_token"])
        else:
            logging.warning(platform_literals.get("azdevops_token_not_found"))
            logging.warning(platform_literals.get("azdevops_download_package_manually"))
    elif source_type == "url":
        paths.download_file(theme_config["source"], destination_path, f"{theme_config['name']}.zip")


def get_themes_path_from_root_path(root_path: str, constants: dict) -> str:
    """ Gets the themes path based on the constants.json from a desired root path

    Args:
        root_path: Full path of the project.
        constants: WordPress constants.
    """

    # Get wordpress path from the constants
    themes_relative_path = constants["paths"]["content"]["themes"]
    themes_path = pathlib.Path.joinpath(pathlib.Path(root_path), themes_relative_path).as_posix()
    logging.info(literals.get("wp_themes_path").format(path=themes_path))

    return themes_path


def install_themes_from_configuration_file(site_configuration: dict, environment_config: dict, global_constants: dict,
                                           root_path: str, skip_partial_dumps: bool, **kwargs):
    """Installs WordPress's theme files (and child themes also) using a site configuration file

    For more information see:
        https://developer.wordpress.org/cli/commands/theme/install/

    Args:
        site_configuration: Parsed site configuration.
        environment_config: Parsed environment configuration.
        global_constants: Global constants ofr WordPress.
        root_path: Path to project root.
        skip_partial_dumps: If True skips database dumps.
    """

    child_theme_config: dict
    parent_theme_config: dict

    # Get data needed in the process
    themes: dict = site_configuration["settings"]["themes"]
    root_path_obj = pathlib.Path(root_path)
    wordpress_path = str(pathlib.Path.joinpath(root_path_obj, global_constants["paths"]["wordpress"]))
    themes_path = pathlib.Path.joinpath(root_path_obj, global_constants["paths"]["content"]["themes"])
    debug_info = environment_config["wp_cli_debug"]

    # Check themes configuration
    if not check_themes_configuration(themes):
        return

    for theme in themes:
        # Check theme configuration
        if not check_theme_configuration(theme):
            continue

        # Get theme path
        theme_path = str(pathlib.Path.joinpath(themes_path, f"{theme['name']}.zip"))
        logging.info(literals.get("wp_theme_path").format(path=theme_path))
        theme["source"] = theme_path

        # Download theme if needed
        if theme["source_type"] in ["url", "feed"]:
            download_wordpress_theme(theme, str(themes_path), **kwargs)

        # Get template for the theme if it has one
        style_content: bytes = devops_toolset.filesystem.zip.read_text_file_in_zip(
            theme_path, f"{theme['name']}/style.css")
        metadata: dict = devops_toolset.filesystem.parsers.parse_theme_metadata(style_content, ["Template", "Version"])
        theme["template"] = metadata["Template"] if "Template" in metadata else None
        theme["version"] = metadata["Version"]

    # Set child theme and parent theme, or just child theme (the one to be activated)
    parent_theme_config, child_theme_config = triage_themes(themes)

    # Install parent theme
    if parent_theme_config:
        wp_cli.install_theme(wordpress_path, parent_theme_config["source"], parent_theme_config["activate"],
                             debug_info, parent_theme_config["name"])
        purge_theme_zip_installation_file_if_generated(parent_theme_config)

    # Install child / single theme
    wp_cli.install_theme(wordpress_path, child_theme_config["source"], child_theme_config["activate"],
                         debug_info, child_theme_config["name"])
    purge_theme_zip_installation_file_if_generated(child_theme_config)

    # Backup database after theme install
    if not skip_partial_dumps:
        database_path = pathlib.Path.joinpath(root_path_obj, global_constants["paths"]["database"])
        core_dump_path_converted = wptools.convert_wp_config_token(
            site_configuration["database"]["dumps"]["theme"], wordpress_path)
        database_core_dump_path = pathlib.Path.joinpath(database_path, core_dump_path_converted)
        wptools.export_database(site_configuration, wordpress_path, database_core_dump_path.as_posix())

    # Warn the user we are skipping the backup dump
    else:
        logging.warning(literals.get("wp_wpcli_export_db_skipping_as_set").format(dump="theme"))


def purge_theme_zip_installation_file_if_generated(theme_config: dict):
    """Purges the ZIP file used for installing the theme if the ZIP has been
    generated (source_type value is not zip in the theme configuration).

    Args:
        theme_config: Parsed theme configuration.
    """

    if theme_config["source_type"] != "zip" and os.path.exists(theme_config["source"]):
        os.remove(theme_config["source"])
        logging.info(literals.get("wp_file_deleted").format(file=theme_config["source"]))


def replace_theme_meta_data_in_package_file(file_path: str, src_theme_configuration: dict):
    """ From the theme configuration, creates a replacements dict and replaces the file_path
    Args:
        file_path: The path of the file to be replaced.
        src_theme_configuration: Dict containing src theme configuration.
    """
    replacements = {"name": src_theme_configuration["source"], "author": {}}
    if "description" in src_theme_configuration:
        replacements["description"] = src_theme_configuration["description"]
    if "tags" in src_theme_configuration:
        replacements["keywords"] = src_theme_configuration["tags"]
    if "author" in src_theme_configuration:
        replacements["author"]["name"] = src_theme_configuration["author"]
    if "author_uri" in src_theme_configuration:
        replacements["author"]["url"] = src_theme_configuration["author_uri"]

    # Open the package_json file
    with open(file_path, 'r') as package_json_file:
        json_data = json.load(package_json_file)
        for key, value in json_data.items():
            if key in replacements:
                json_data[key] = replacements[key]

    with open(file_path, 'w') as package_json_file:
        json.dump(json_data, package_json_file, indent=2)


def replace_theme_meta_data_in_scss_file(file_path: str, src_theme_configuration: dict):
    """ From the theme configuration, creates a replacements dict and replaces the file_path
    Args:
        file_path: The path of the file to be replaced.
        src_theme_configuration: Dict containing src theme configuration.
        """

    # Create a dict of replacements for the style.scss file based on the content of the src theme configuration
    replacements = {"Theme Name": src_theme_configuration["name"], "Text Domain": src_theme_configuration["source"]}
    if "description" in src_theme_configuration:
        replacements["Description"] = src_theme_configuration["description"]
    if "uri" in src_theme_configuration:
        replacements["Theme URI"] = src_theme_configuration["uri"]
    if "author" in src_theme_configuration:
        replacements["Author"] = src_theme_configuration["author"]
    if "author_uri" in src_theme_configuration:
        replacements["Author URI"] = src_theme_configuration["author_uri"]
    if "tags" in src_theme_configuration:
        replacements["Tags"] = ", ".join(src_theme_configuration["tags"])

    replace_theme_meta_data(file_path, replacements, wp_constants.theme_metadata_parse_regex)


def replace_theme_meta_data(path: str, replacements: dict, regex_str: str):
    """ Opens the file path and rewrites it replacing matches from the replacements dict
    Args:
        path: The file path to be replaced.
        replacements: The replacements content.
        regex_str: The regex string used to match and replace the replacements dict.
    """

    replace_file = open(path, 'r', newline='\n')
    file_content = replace_file.read()

    for key, replacement in replacements.items():
        # Build the regex to retrieve data: This regex would match the lines containing [key]:value.
        # For example, in the string Author: Ahead Labs, S.L -> $1 = Author and $2 = Ahead Labs, S.L
        regex = f'({key}){regex_str}'

        # Search with regex
        matches = re.search(regex, file_content)

        if matches is not None and matches.group(1):
            # Build the replacement string
            target = f'{matches.group(1)}: {replacement}'
            # Apply the replacement
            file_content = file_content.replace(matches.group(0), target)

    # Write replaced content to the file
    with open(path, 'w', newline='\n') as scss_file:
        scss_file.write(file_content)


def replace_theme_slug_in_functions_php(file_path: str, src_theme_configuration: dict):
    """ From the theme configuration, creates a replacements dict and replaces the file_path
    Args:
        file_path: The path of the file to be replaced.
        src_theme_configuration: Dict containing src theme configuration.
    """
    theme_slug = src_theme_configuration["source"]
    core_php_file = open(file_path, 'r')
    file_content = core_php_file.read()

    file_content = re.sub(wp_constants.functions_php_mytheme_regex, theme_slug, file_content)

    # Write replaced content to the file
    with open(file_path, 'w', newline='\n') as core_php_file:
        core_php_file.write(file_content)


def set_theme_metadata(root_path: str, src_theme_configuration: dict):
    """ Replaces theme meta-data values on required theme files
    Args:
        root_path: The path of the project
        src_theme_configuration: The src theme configuration to be set.
    """

    constants = wptools.get_constants()
    theme_slug = src_theme_configuration["source"]

    # Get scss file path and content from the theme slug
    themes_path = pathlib.Path.joinpath(pathlib.Path(root_path), constants["paths"]["content"]["themes"])

    # Replace meta-data obtained inside theme .scss
    scss_file_path = str(pathlib.Path.joinpath(themes_path, theme_slug, "src", "assets", "css", "style.scss"))
    replace_theme_meta_data_in_scss_file(scss_file_path, src_theme_configuration)

    # Replace meta-data on the package.json file
    package_json_file_path = str(pathlib.Path.joinpath(themes_path, theme_slug, "package.json"))
    replace_theme_meta_data_in_package_file(package_json_file_path, src_theme_configuration)

    # Replace theme slug on the functions_core.php
    functions_php_file_path = str(pathlib.Path.joinpath(themes_path, theme_slug, "src", "functions_php", "_core.php"))
    replace_theme_slug_in_functions_php(functions_php_file_path, src_theme_configuration)

    # Update project.xml name
    project_xml_path = pathlib.Path.joinpath(pathlib.Path(root_path), "project.xml").as_posix()
    devops_toolset.filesystem.tools.update_xml_file_entity_text("./name", theme_slug, project_xml_path)


def start_basic_theme_structure(path: str, theme_name: str, structure_file_path: str = None) -> None:
    """ Creates a basic structure of a wordpress development theme based on its default theme-structure.json

        Args:
            path: Full path where the structure will be created
            theme_name: Name of the theme to be used to create the root folder
            structure_file_path: Optional parameter containing the structure file path to be used. If ignored, the
            default structure file will be used.
        """

    # Parse theme structure configuration
    if pathlib.Path.exists(pathlib.Path(structure_file_path)):
        theme_structure = wptools.get_site_configuration(structure_file_path)
        logging.info(literals.get("wp_theme_structure_creating_from_file").format(theme_name=theme_name,
                                                                                  file_name=structure_file_path))
    else:
        theme_structure = wptools.get_project_structure(
            devops_platforms_constants.Urls.DEFAULT_WORDPRESS_DEVELOPMENT_THEME_STRUCTURE)
        logging.info(literals.get("wp_theme_structure_creating_from_default_file").format(
            resource=devops_platforms_constants.Urls.DEFAULT_WORDPRESS_DEVELOPMENT_THEME_STRUCTURE))

    project_starter = BasicStructureStarter()

    # Change the main folder's name of the theme to the theme_name
    theme_structure["items"][0]["name"] = theme_name

    # Purge .gitkeep
    git_tools.purge_gitkeep(pathlib.Path(path).as_posix())

    # Iterate through every item recursively
    for item in theme_structure["items"]:
        project_starter.add_item(item, path)

    logging.info(literals.get("wp_created_theme_structure").format(theme_name=theme_name))


def triage_themes(themes: dict) -> (dict, dict):
    """triages themes to determine which must be installed and activated.

    Args:
        themes: Themes configuration.

    Return:
        Tuple with parent and child theme configuration.
    """
    child = None
    parent = None
    parent_guess: str = ""

    for theme in themes:
        if not theme["activate"]:
            parent_guess = theme["name"]
        if theme["activate"]:
            child = theme
            if theme["template"]:
                parent_guess = theme["template"]

        if not theme["activate"] and not theme["template"] and theme["name"] == parent_guess:
            parent = theme

    return parent, child


if __name__ == "__main__":
    help(__name__)



"""DevOps common commands"""

from devops_toolset.core.ValueDictsBase import ValueDictsBase
from devops_toolset.core.app import App

app: App = App()


class Commands(ValueDictsBase):
    """Commands for common DevOps code."""

    # Add your commands here
    _commands = {
        "echo": "echo {variable}",
    }



"""Initializes gettext"""

import gettext
from devops_toolset.core.settings import Settings


def setup(settings: Settings):
    """Sets up translations for the selected language"""
    gettext.translation("base", localedir=settings.locales_path, languages=[settings.language]).install()



"""Contains paths-related operations."""

import logging
import os
import pathlib
import requests
import shutil
from devops_toolset.core.app import App
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.filesystem.Literals import Literals as FileSystemLiterals
from devops_toolset.filesystem.constants import Directions, FileNames, FileType
from typing import List, Tuple, Union
from urllib.parse import urlparse

app: App = App()
platform_specific = app.load_platform_specific("environment")
literals = LiteralsCore([FileSystemLiterals])


# noinspection PyTypeChecker
def download_file(url: str, destination: str, save_as: str = None, headers: dict = None,
                  file_type: FileType = FileType.BINARY) -> tuple:
    """Downloads a file from a URL.

    Args:
        url: Where to download the file from.
        destination: Path to the directory where the file will be downloaded.
        save_as: File name to save the downloaded file as.
        headers: Authentication headers.
        file_type: The type of the file. Defaults to BINARY.

    Returns:
        Tuple with (file name, file path)
    """

    if not os.path.isdir(destination):
        raise ValueError(literals.get("fs_not_dir"))

    destination_path = pathlib.Path(destination)
    file_name = save_as if save_as else get_file_name_from_url(url)
    full_destination_path = pathlib.Path.joinpath(destination_path, file_name)

    response = requests.get(url, headers=headers) if headers else requests.get(url)

    file_mode: str = f"w{file_type.value}"
    with open(full_destination_path, file_mode) as file:
        file.write(response.content)

    return file_name, full_destination_path


def files_exist(path: str, file_names: List[str]) -> List[Tuple[str, bool]]:
    """Determines if every file path in the list exists in the specified path.

    Args:
        path: Path where files will be checked.
        file_names: List of file names to be checked.

    Returns:
         List of tuples where each element contains the path evaluated (glob if
         not found) and a boolean value: True if path exists; False if it
         doesn't.
    """

    result = []

    for file_name in file_names:
        files = sorted(pathlib.Path(path).rglob(file_name))
        if len(files) == 0:
            result.append((file_name, False))
        elif len(files) > 1:
            for file in files:
                result.append((file, True))
        else:
            result.append((files[0], True))

    return result


def files_exist_filtered(path: str, filter_by: bool, file_names: List[str]) -> List[str]:
    """Returns a filtered list, only with values that meet the condition.

    Args:
        path: Path where files will be checked.
        filter_by: Returns the value[0] that meets the criteria on value[1].
        file_names: List of file names to be checked.

    Returns:
        List of strings that meet the filter.
    """

    unfiltered_list = files_exist(path, file_names)

    filtered_list = []
    for value in unfiltered_list:
        if filter_by == value[1]:
            filtered_list.append(value[0])

    return filtered_list


def get_file_name_from_url(url: str) -> str:
    """Returns the file name from a URL.

    Args:
        url: URL to be parsed.

    Returns:
        File name.
    """

    parsed = urlparse(url)
    return os.path.basename(parsed.path)


def get_file_path_from_pattern(path: str, pattern: str, recursive: bool = False) -> Union[List[str], str, None]:
    """Gets the file path from a file name pattern.

    Args:
        path: Where to look for.
        pattern: glob pattern of the file name to be found.
        recursive: If True the search will be recursive.

    Returns:
        None if no file or more than one is found, path to file if one found.
    """

    if recursive:
        files = sorted(pathlib.Path(path).rglob(pattern))
    else:
        files = sorted(pathlib.Path(path).glob(pattern))

    if len(files) == 0:
        return None
    elif len(files) > 1:
        file_list = []
        for file in files:
            file_list.append(str(file))
        return file_list
    else:
        return str(files[0])


def get_file_paths_in_tree(starting_path: str, glob: str) -> List[pathlib.Path]:
    """Gets a list with the paths to the descendant files that match the glob pattern.

    Args:
        starting_path: Path to start the seek from.
        glob: glob pattern to match the files that should be found.

    Returns:
        List with the paths to the files that match.
    """

    paths = []

    for guess_path in pathlib.Path(starting_path).rglob(glob):
        paths.append(guess_path)

    return paths


def get_filepath_in_tree(
        base_path: pathlib.Path, seed: str, direction: Directions = Directions.ASCENDING) -> pathlib.PurePath:
    """Gets path to the directory containing the file.

    Args:
        base_path: Path where the search starts from.
        seed: File or directory name (not path to file) that should be found.
        direction: The direction of the seek (ascending by default).

    Returns:
        Path to the directory or None if path not found.
    """

    current_path = base_path
    path_to_file = None

    if direction == Directions.ASCENDING:
        for i in range(len(current_path.parents)):
            guess_path = pathlib.Path.joinpath(current_path.parents[i], seed)
            if pathlib.Path(guess_path).exists():
                path_to_file = pathlib.Path(guess_path).parent
                break
    else:
        for guess_path in current_path.parent.rglob(seed):
            if pathlib.Path(guess_path).exists():
                path_to_file = pathlib.Path(guess_path).parent
                break
            else:
                path_to_file = None

    return path_to_file


def get_project_root(base_path: pathlib.Path, seed: str = FileNames.PROJECT_FILE) -> str:
    """Gets the project root directory path.

    Args:
        base_path: Path where the search starts from.
        seed: What file or directory to look for.

    Returns:
        Path to the project root directory or None if path not found."""

    return str(get_filepath_in_tree(base_path, seed))


def is_empty_dir(path: str = None) -> bool:
    """Checks if the current path is an empty directory

       Args:
           path: Path string to be analyzed

       Returns:
           True if path is an empty directory
       """

    path_object = pathlib.Path(path)
    files_inside_path = filter(lambda x: pathlib.Path.is_dir(pathlib.Path(x)) is False, os.listdir(path_object))
    try:
        min(files_inside_path)
    except ValueError:
        return True
    return False


def is_valid_path(path: Union[str, None] = None, check_existence: bool = False) -> bool:
    """Checks if it is a valid path.

    Args:
        path: Path string to be analyzed.
        check_existence: If True, it checks that the path exists.

    Returns:
        True if path is valid an exists.
    """

    if path is None or path.strip() == "":
        logging.info(literals.get("fs_file_path_not_valid"))
        return False

    if check_existence and not os.path.exists(path):
        logging.info(literals.get("fs_file_path_does_not_exist"))
        return False

    return True


def move_files(origin: str, destination: str, glob: str, recursive: bool = False):
    """Moves one or more files from one path to another.

    Args:
        origin: Path where files and directories should be found.
        destination: Path where files and directories will be moved.
        glob: Files to be matched to be moved.
        recursive: If True files are searched recursively
    """

    destination_directory_path = pathlib.Path(destination)

    if recursive:
        # Convert file list to a list
        files = [*pathlib.Path(origin).rglob(glob)]
    else:
        # Keep file list in a generator
        files = pathlib.Path(origin).glob(glob)

    # Iterate through files to move them
    for origin_file_path in files:
        # Get paths related to the file
        origin_directory_path: str = pathlib.Path(os.path.commonprefix([origin, origin_file_path])).as_posix()
        relative_path: str = origin_file_path.as_posix().removeprefix(origin_directory_path).lstrip("/")
        destination_file_path: str = pathlib.Path.joinpath(destination_directory_path, relative_path).as_posix()

        # Move file
        logging.info(literals.get("fs_file_moving").format(
            origin_file_path=origin_file_path,
            destination_file_path=destination_file_path
        ))
        shutil.move(origin_file_path, destination_file_path)


if __name__ == "__main__":
    help(__name__)



""" Dotnet utilities """
import devops_toolset.filesystem.parsers as parsers
import devops_toolset.tools.git_flow as gitflow
import devops_toolset.tools.git as git
import logging
import os.path
import pathlib
import re
from devops_toolset.core.app import App
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.project_types.dotnet.Literals import Literals as DotnetLiterals
from devops_toolset.core.CommandsCore import CommandsCore
from devops_toolset.project_types.dotnet.commands import Commands as DotnetCommands


app: App = App()
platform_specific = app.load_platform_specific("environment")
literals = LiteralsCore([DotnetLiterals])
commands = CommandsCore([DotnetCommands])


def get_appsettings_environments(csproj_directory_path: str, include_development: bool = False) -> list[str]:
    """Gets a list of all environments configured in the solution.

    Args:
        csproj_directory_path: Path to directory that contains the csproj file.
        include_development: If True, Development/Dev is included in the list.

    Returns:
        List with all appsettings.*.json environment names.
    """

    path = pathlib.Path(csproj_directory_path)
    glob_pattern = "appsettings.*.json"
    regex_pattern = "^appsettings\.([A-Za-z]+)\.json$"
    environments: list[str] = []

    logging.info(literals.get("dotnet_ef_utils_getting_appsettings_files").format(path=path))

    files = list(path.glob(glob_pattern))
    logging.info(literals.get("dotnet_ef_utils_appsettings_files_matched").format(files=files))
    for file in files:
        filename = os.path.basename(file)

        match = re.search(regex_pattern, filename)
        if match is None:
            logging.info(literals.get("dotnet_ef_utils_appsettings_no_environment_matched").format(filename=filename))
            continue

        logging.info(literals.get("dotnet_ef_utils_appsettings_environment_matched")
                     .format(filename=filename, environment=match.groups()[0]))
        environment = match.groups()[0]
        if (environment == "Development" or environment == "Dev") and not include_development:
            logging.info(literals.get("dotnet_ef_utils_appsettings_dev_environment_skipped"))
            continue
        environments.append(environment)

    return environments


def get_csproj_project_version(csproj_path: str, environment_variable_name: str = "DT_PROJECT_VERSION") -> str:
    """ Gets the version number from a .csproj file

    Arguments:
        csproj_path: Path to the .csproj file.
        environment_variable_name: Name of the environment variable to be
            created. Defaults to "DT_PROJECT_VERSION".

    Returns:
        The version number defined in the .csproj file.
    """

    version = parsers.get_xml_file_entity_text("./PropertyGroup/Version", csproj_path)

    version_environment_variable = {environment_variable_name: version["Version"]}
    platform_specific.create_environment_variables(version_environment_variable)

    return version["Version"]


def git_tag(commit_name: str, tag_name: str, branch: str):
    """ Does a git tag over a checkout branch's commit
        Args:
            commit_name: Name of the commit.
            Git will need the checksum name, or part of it.
            F.I: If the commit name is 9fceb02d0ae598e95dc970b74767f19372d61af8, the checksum will be 9fceb02
            tag_name: Name of the tag to be added
            branch: The simplified name of the git branch
    """
    if gitflow.is_branch_suitable_for_tagging(branch):
        logging.info(literals.get("dotnet_git_tag")
                     .format(tag_name=tag_name, commit_name=commit_name, branch_name=branch))
        git.git_tag_add(tag_name, commit_name)


if __name__ == "__main__":
    help(__name__)



"""node module literals"""

from devops_toolset.core.ValueDictsBase import ValueDictsBase
from devops_toolset.core.app import App

app: App = App()


class Literals(ValueDictsBase):
    """ValueDicts for the node module."""

    _info = {
        "npm_run_before": _("Running npm task: {task}..."),
        "npm_run_after": _("npm task: {task} has completed successfully."),
        "npm_install_before": _("Npm install launched. This may take a while..."),
        "npm_install_after": _("Npm install completed successfully.")
    }
    _errors = {
        "npm_run_error": _("An error has occurred while running task {task}. See above log for more details."),
        "npm_install_error": _("Npm install has encountered an error. Please check above logs for more details.")
    }



"""Utils for managing the gettext API

If called with no arguments it compiles .po files to .mo format using Linux gettext

Args:
    --py: If present it will use Python pygettext.py and msgfmt.py instead of Linux's
    --generate-pot: If present it generates the .pot file
    --compile: If present it compiles .po files to .mo format
    --merge: If present it merges the .pot file with the existing po files.
"""

from typing import List
import argparse
import devops_toolset.core.app
import os
import pathlib
import devops_toolset.tools.cli as tools_cli
import shutil

parser = argparse.ArgumentParser()
parser.add_argument("--py", action="store_true")
parser.add_argument("--generate-pot", action="store_true")
parser.add_argument("--compile", action="store_true")
parser.add_argument("--skip-i18n", action="store_true")
parser.add_argument("--merge", action="store_true")
args, args_unknown = parser.parse_known_args()

app: devops_toolset.core.app.App = devops_toolset.core.app.App(args.skip_i18n)

base_pot_file_name = "base.pot"


def get_files(starting_path: str, glob: str) -> List[pathlib.Path]:
    """Gets a list with the paths to the descendant files that match the glob pattern.

    Args:
        starting_path: Path to start the seek from.
        glob: glob pattern to match the files that should be found.

    Returns:
        List with the paths to the files that match.
    """

    return list(pathlib.Path(starting_path).rglob(glob))


def generate_pot_file():
    """Generates the .pot file from the strings found in the code"""

    pot_file = pathlib.Path.joinpath(app.settings.locales_path, base_pot_file_name)

    if pathlib.Path(pot_file).exists():
        os.remove(str(pot_file))

    files = get_files(str(app.settings.root_path), "**/*.py")

    script = "pygettext.py" if args.py else "xgettext --from-code=utf-8"

    command = f"{script} -d base -o {str(pot_file)} {' '.join(map(str, files))}"

    tools_cli.call_subprocess(command)

    distribute_pot()


def compile_po_files():
    """Compiles .po files to .mo files"""

    paths = get_files(str(app.settings.locales_path), "**/*.po")

    for file in paths:
        po_file = pathlib.Path(file)
        mo_file = po_file.with_suffix(".mo")

        if pathlib.Path(mo_file).exists():
            os.remove(mo_file)

        py = ".py" if args.py else ""
        command = f"msgfmt{py} -o {mo_file} {file}"

        tools_cli.call_subprocess(command)


def distribute_pot():
    """ Copies the generated pot file inside LC_MESSAGES of every language, and renames the file as a .po """

    pot_file = pathlib.Path.joinpath(app.settings.locales_path, base_pot_file_name)

    if not pathlib.Path(pot_file).exists():
        # Not pot to distribute, nothing to do here
        return

    # This will get the first level subdirectories from the "locales" location
    destination_paths = next(os.walk(app.settings.locales_path))[1]

    for destination_path in destination_paths:
        destination_path = pathlib.Path.joinpath(app.settings.locales_path, pathlib.Path(destination_path),
                                                 "LC_MESSAGES", "base.po")
        if pathlib.Path(destination_path).exists():
            os.remove(destination_path)

        shutil.copy(pot_file, destination_path)


def merge_pot_file():
    """Merges a .pot file with existing .po files
    # https://www.gnu.org/software/gettext/manual/html_node/msgmerge-Invocation.html#msgmerge-Invocation """

    paths = get_files(str(app.settings.locales_path), "**/*.po")
    pot_file = pathlib.Path.joinpath(app.settings.locales_path, base_pot_file_name)

    if not pathlib.Path(pot_file).exists():
        generate_pot_file()

    for file in paths:
        po_file = pathlib.Path(file)
        command = f"msgmerge -U {po_file} {pot_file}"
        tools_cli.call_subprocess(command)


if __name__ == "__main__":
    if not pathlib.Path(app.settings.locales_path).is_dir():
        raise ValueError("locale_path parameter must be a directory. Check app settings.")
    if args.generate_pot:
        generate_pot_file()
    if args.compile:
        compile_po_files()
    if args.merge:
        merge_pot_file()



"""Commands for the package."""

from devops_toolset.core.ValueDictsBase import ValueDictsBase
from devops_toolset.core.app import App

app: App = App()


class CommandsCore(ValueDictsBase):
    """Core literals for the package.


    1. Create an instance of the CommandsCore class (this class).
    2. Pass as a parameter list all Commands classes you want to include.

        from devops_toolset.core.CommandsCore import CommandsCore
        from wordpress.commands import Commands as WpCommands

        commands = CommandsCore([WpCommands])

    3. Get a command by it's key.

        commands.get("wp_info")
    """

    # Add your core command dictionaries here
    _commands = {}



""" This script will hold tools to install and update software in a remote linux host """
from devops_toolset.core.app import App
from devops_toolset.core.CommandsCore import CommandsCore
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.project_types.linux.commands import Commands as LinuxCommands
from devops_toolset.project_types.linux.Literals import Literals as LinuxLiterals
from devops_toolset.tools import cli
import argparse
import json

app: App = App()
literals = LiteralsCore([LinuxLiterals])
commands = CommandsCore([LinuxCommands])


def check_and_update_instance_software(software_config: dict):
    """ Checks all software included on the software config dict. If a software doesn't exist or
    needs to be updated, it'll execute the neccessary sub-commands to install/update software required
    Args:
        software_config: Dict containing software configuration of the instance
    """
    for package, version in software_config.items():
        install_package(package, version)


def check_package_installed(package: str) -> bool:
    """ Checks if a concrete package is installed. Returns true if present, false otherwise.
    Args:
        package: The name of the package to be checked: For example: python3,
    """
    result = cli.call_subprocess_with_result(commands.get("deb_which").format(
        package=package))
    return result is not None


def convert_version_parameter(value: str) -> str:
    """ Converts a boolean value to a --silent string."""
    if value is not None and value != "latest":
        return f"--version \"{value}\""
    return ""


def install_package(package: str, version: str = None):
    """ Installs a package
        Omitting version will only check for latest version (by not targeting a concrete version)
     Args:
         package: The package that will be installed
         version: Target a concrete version
     """
    version = convert_version_parameter(version)

    cli.call_subprocess(commands.get("deb_package_install").format(package=package, version=version),
                        log_before_process=[literals.get("deb_package_install_pre").format(package=package)],
                        log_after_err=[literals.get("deb_package_install_err").format(package=package)],
                        log_after_out=[literals.get("deb_package_install_post").format(package=package)])


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("instance-config-path", action=tools.argument_validators.PathValidator)
    args, args_unknown = parser.parse_known_args()
    with open(args.instance_config_path) as instance_file:
        instance_dict = json.load(instance_file)
        check_and_update_instance_software(instance_dict["software-packages"])






"""Environment-related functionality for Azure DevOps"""
import devops_toolset.devops_platforms.common
from devops_toolset.core.app import App
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.devops_platforms.Literals import Literals as CommonLiterals
from devops_toolset.devops_platforms.azuredevops.Literals import Literals as AzureDevOpsLiterals
from enum import Enum
import logging
import sys

app: App = App()
literals = LiteralsCore([CommonLiterals, AzureDevOpsLiterals])


class ResultType(Enum):
    """Result types for a task"""
    success = "Succeeded"
    partial_success = "SucceededWithIssues"
    fail = "Failed"


def create_environment_variables(key_value_pairs: dict):
    """Creates environment variables

    Args:
        key_value_pairs: Key-value pair dictionary
    """

    for key, value in key_value_pairs.items():
        logging.info(literals.get("platform_created_ev").format(key=key, value=value))
        sys.stdout.write(f"##vso[task.setvariable variable={key}]{value}\n")


def end_task(result_type: ResultType):
    """Ends the current task

    Args:
        result_type: Result type of the task
    """

    sys.stdout.write(f"##vso[task.complete result={result_type.value};]DONE\n")


def get_platform_variable_dict() -> dict:
    """Gets all keys for the environment variables defined by default in the
    platform. Values are replaced automatically by Azure DevOps before the
    script is executed.

    Note: Since Azure DevOps Pipelines expands variables only on the YAML file,
        you should copy and paste this dict there because calling this function
        will not expand values.

    More info at:
        https://docs.microsoft.com/en-us/azure/devops/pipelines/build/variables

    Returns
        List with all variables
    """

    return {
        "Agent.BuildDirectory": "$(Agent.BuildDirectory)",
        "Agent.ContainerMapping": "$(Agent.ContainerMapping)",
        "Agent.HomeDirectory": "$(Agent.HomeDirectory)",
        "Agent.Id": "$(Agent.Id)",
        "Agent.JobName": "$(Agent.JobName)",
        "Agent.JobStatus": "$(Agent.JobStatus)",
        "Agent.MachineName": "$(Agent.MachineName)",
        "Agent.Name": "$(Agent.Name)",
        "Agent.OS": "$(Agent.OS)",
        "Agent.OSArchitecture": "$(Agent.OSArchitecture)",
        "Agent.TempDirectory": "$(Agent.TempDirectory)",
        "Agent.ToolsDirectory": "$(Agent.ToolsDirectory)",
        "Agent.WorkFolder": "$(Agent.WorkFolder)",
        "Build.ArtifactStagingDirectory": "$(Build.ArtifactStagingDirectory)",
        "Build.BuildId": "$(Build.BuildId)",
        "Build.BuildNumber": "$(Build.BuildNumber)",
        "Build.BuildUri": "$(Build.BuildUri)",
        "Build.BinariesDirectory": "$(Build.BinariesDirectory)",
        "Build.ContainerId": "$(Build.ContainerId)",
        "Build.DefinitionName": "$(Build.DefinitionName)",
        "Build.DefinitionVersion": "$(Build.DefinitionVersion)",
        "Build.QueuedBy": "$(Build.QueuedBy)",
        "Build.QueuedById": "$(Build.QueuedById)",
        "Build.Reason": "$(Build.Reason)",
        "Build.Repository.Clean": "$(Build.Repository.Clean)",
        "Build.Repository.Git.SubmoduleCheckout": "$(Build.Repository.Git.SubmoduleCheckout)",
        "Build.Repository.ID": "$(Build.Repository.ID)",
        "Build.Repository.LocalPath": "$(Build.Repository.LocalPath)",
        "Build.Repository.Name": "$(Build.Repository.Name)",
        "Build.Repository.Provider": "$(Build.Repository.Provider)",
        "Build.Repository.Tfvc.Workspace": "$(Build.Repository.Tfvc.Workspace)",
        "Build.Repository.Uri": "$(Build.Repository.Uri)",
        "Build.RequestedFor": "$(Build.RequestedFor)",
        "Build.RequestedForEmail": "$(Build.RequestedForEmail)",
        "Build.RequestedForId": "$(Build.RequestedForId)",
        "Build.SourceBranch": "$(Build.SourceBranch)",
        "Build.SourceBranchName": "$(Build.SourceBranchName)",
        "Build.SourcesDirectory": "$(Build.SourcesDirectory)",
        "Build.SourceVersion": "$(Build.SourceVersion)",
        "Build.SourceVersionMessage": "$(Build.SourceVersionMessage)",
        "Build.StagingDirectory": "$(Build.StagingDirectory)",
        "Build.SourceTfvcShelveset": "$(Build.SourceTfvcShelveset)",
        "Build.TriggeredBy.BuildId": "$(Build.TriggeredBy.BuildId)",
        "Build.TriggeredBy.BuildNumber": "$(Build.TriggeredBy.BuildNumber)",
        "Build.TriggeredBy.DefinitionId": "$(Build.TriggeredBy.DefinitionId)",
        "Build.TriggeredBy.DefinitionName": "$(Build.TriggeredBy.DefinitionName)",
        "Build.TriggeredBy.ProjectID": "$(Build.TriggeredBy.ProjectID)",
        "Checks.StageAttempt": "$(Checks.StageAttempt)",
        "Common.TestResultsDirectory": "$(Common.TestResultsDirectory)",
        "Environment.Id": "$(Environment.Id)",
        "Environment.Name": "$(Environment.Name)",
        "Environment.ResourceId": "$(Environment.ResourceId)",
        "Environment.ResourceName": "$(Environment.ResourceName)",
        "Pipeline.Workspace": "$(Pipeline.Workspace)",
        "Strategy.CycleName": "$(Strategy.CycleName)",
        "Strategy.Name": "$(Strategy.Name)",
        "System.AccessToken": "$(System.AccessToken)",
        "System.Debug": "$(System.Debug)",
        "System.CollectionId": "$(System.CollectionId)",
        "System.CollectionUri": "$(System.CollectionUri)",
        "System.DefaultWorkingDirectory": "$(System.DefaultWorkingDirectory)",
        "System.DefinitionId": "$(System.DefinitionId)",
        "System.HostType": "$(System.HostType)",
        "System.JobAttempt": "$(System.JobAttempt)",
        "System.JobDisplayName": "$(System.JobDisplayName)",
        "System.JobId": "$(System.JobId)",
        "System.JobName": "$(System.JobName)",
        "System.PhaseAttempt": "$(System.PhaseAttempt)",
        "System.PhaseDisplayName": "$(System.PhaseDisplayName)",
        "System.PhaseName": "$(System.PhaseName)",
        "System.StageAttempt": "$(System.StageAttempt)",
        "System.StageDisplayName": "$(System.StageDisplayName)",
        "System.StageName": "$(System.StageName)",
        "System.PullRequest.IsFork": "$(System.PullRequest.IsFork)",
        "System.PullRequest.PullRequestId": "$(System.PullRequest.PullRequestId)",
        "System.PullRequest.PullRequestNumber": "$(System.PullRequest.PullRequestNumber)",
        "System.PullRequest.SourceBranch": "$(System.PullRequest.SourceBranch)",
        "System.PullRequest.SourceRepositoryURI": "$(System.PullRequest.SourceRepositoryURI)",
        "System.PullRequest.TargetBranch": "$(System.PullRequest.TargetBranch)",
        "System.TeamFoundationCollectionUri": "$(System.TeamFoundationCollectionUri)",
        "System.TeamProject": "$(System.TeamProject)",
        "System.TeamProjectId": "$(System.TeamProjectId)",
        "TF_BUILD": "$(TF_BUILD)",
    }


def log_environment_variables(platform_keys: dict):
    """Logs all environment variables for this platform and process.

    Args:
        platform_keys: dict with key-value pairs of platform variables.
    """

    devops_toolset.devops_platforms.common.log_environment_variables(platform_keys)


if __name__ == "__main__":
    help(__name__)



"""Contains argument validators"""

import argparse
import devops_toolset.filesystem.paths as paths

from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.tools.Literals import Literals as ToolsLiterals

literals = LiteralsCore([ToolsLiterals])


class PathValidator(argparse.Action):
    """Validates a path"""
    def __init__(self, option_strings, dest, nargs=None, **kwargs):
        super(PathValidator, self).__init__(option_strings, dest.replace("-", "_"), **kwargs)

    def __call__(self, parent_parser, namespace, values, option_string=None):
        setattr(namespace, self.dest, values)

        if not paths.is_valid_path(values):
            raise ValueError(literals.get("val_path_argument_not_valid").format(argument=self.dest))



""" Contains Angular utilities """

from devops_toolset.core.app import App
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.project_types.angular.Literals import Literals as AngularLiterals
from devops_toolset.core.CommandsCore import CommandsCore
from devops_toolset.project_types.angular.commands import Commands as AngularCommands

import devops_toolset.filesystem.parsers as parsers
import logging
import devops_toolset.filesystem.tools as filesystem

app: App = App()
platform_specific = app.load_platform_specific("environment")
literals = LiteralsCore([AngularLiterals])
commands = CommandsCore([AngularCommands])


def get_packagejson_project_version(packagejson_path: str, create_environment_variable: bool = True) -> str:
    """Gets the version number from a package.json file

    Arguments:
        packagejson_path: Path to the package.json file.
        create_environment_variable: If True, it creates an environment
            variable with the version value.

    Returns:
        The version number defined in the package.json file.
    """

    package_json: dict = parsers.parse_json_file(packagejson_path)
    version = package_json["version"]

    if create_environment_variable:
        version_environment_variable = {"PROJECT_VERSION": version}
        platform_specific.create_environment_variables(version_environment_variable)

    logging.info(literals.get("angular_project_version").format(version=version))

    return version


def set_project_version_in_json_file(packagejson_path: str, destination_file_path: str,
                                     create_environment_variable: bool = True):
    """Gets the project version from the package.json file and sets its value
        in a custom json file.

    Args:
        packagejson_path: Path to the package.json file.
        destination_file_path: Path to the JSON file.
        create_environment_variable: If True, creates an environment variable
            with the project version.
    """

    version: str = get_packagejson_project_version(packagejson_path, create_environment_variable)
    filesystem.update_json_file_key_text(["version"], version, destination_file_path)


if __name__ == "__main__":
    help(__name__)



"""Provides tools for managing the Azure Database service"""

from devops_toolset.core.app import App
from devops_toolset.core.CommandsCore import CommandsCore
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.project_types.azure.commands import Commands as AzureCommands, Log as Log
from devops_toolset.project_types.azure.Literals import Literals as AzureLiterals
from devops_toolset.tools import cli

import devops_toolset.filesystem.tools
import devops_toolset.project_types.azure.common as common
import json
import logging
import re

app: App = App()
literals = LiteralsCore([AzureLiterals])
commands = CommandsCore([AzureCommands])
rule_name_fix_regex = "[^A-Za-z0-9-_]"


def add_mysql_flexible_server_firewall_rule(server_name: str, resource_group: str, rule_name: str,
                                            ip_address: str, end_ip_address: [str, None] = None,
                                            log: Log = Log.OFF) -> dict:
    """Adds a firewall rule to MySQL Flexible server.

    Args:
        server_name: Name of the server we target.
        resource_group: Azure resource group that the server belongs to.
        rule_name: Name of the rule to be created.
        ip_address: Unique IP address to add to the rule if we are not adding a
            range. In case of adding a range, this will be the starting IP
            address of the range.
        end_ip_address: End IP address of the range. If we are adding a unique
            IP address this value must be None.
        log: Log level from OFF, VERBOSE or DEBUG.

    Returns:
        Operation result
    """

    az_command: str = commands.get("azure_cli_db_mysql_flexible_server_firewall_rule_create").format(
        server_name=server_name,
        resource_group=resource_group,
        rule_name=re.sub(rule_name_fix_regex, "-", rule_name),
        start_ip_address=ip_address,
        end_ip_address=ip_address if end_ip_address is None else end_ip_address,
        log="" if log == Log.OFF else f"--{log.name.lower()}"
    )
    logging.info(literals.get("azure_cli_executing_command").format(command=az_command))

    result: str = cli.call_subprocess_with_result(az_command)
    logging.info(literals.get("azure_cli_command_output").format(output=result))

    return json.loads(result)


def execute_mysql_flexible_server_sql_script(admin_user: str, admin_password: str, server_name: str, database_name: str,
                                             file_path: [str, None] = None, query: [str, None] = None,
                                             log: Log = Log.OFF) -> [str, None]:
    """Executes a script (file) / query (text) against MySQL Flexible server.

    Args:
        admin_user: Admin user for the server.
        admin_password: Password for the admin user.
        server_name: Name of the server.
        database_name: Name of the database.
        file_path: Path to the SQL file to be executed.
        query: Query text to be executed. If file_path is not None, query will
            be ignored.
        log: Log level from OFF, VERBOSE or DEBUG.
    """

    # Check that parameters are correct
    if file_path is None and query is None:
        raise ValueError(literals.get("azure_cli_db_mysql_flexible_server_execute_file_query_parameters_error"))

    # Install rdbms-connect Azure CLI extension if not already installed
    az_extension: str = "rdbms-connect"
    if not common.is_cli_extension_installed(az_extension):
        az_command: str = commands.get("azure_cli_extension_add").format(name=az_extension)
        logging.info(literals.get("azure_cli_executing_command").format(command=az_command))
        cli.call_subprocess(az_command)

    # Strip UTF-8 BOM from script file
    if file_path is not None:
        devops_toolset.filesystem.tools.strip_utf8_bom_character_from_file(file_path)

    # Compose and execute command
    az_command: str = commands.get("azure_cli_db_mysql_flexible_server_execute").format(
        server_name=server_name,
        admin_user=admin_user,
        admin_password=admin_password,
        database_name=database_name,
        file_path=f"-f \"{file_path}\"" if file_path is not None else "",
        query=f"-q \"{query}\"" if query is not None and file_path is None else "",
        log="" if log == Log.OFF else f"--{log.name.lower()}"
    )
    logging.info(literals.get("azure_cli_executing_command").format(command=az_command))

    result: str = cli.call_subprocess_with_result(az_command)
    logging.info(literals.get("azure_cli_command_output").format(output=result))


def remove_mysql_flexible_server_firewall_rule(server_name: str, resource_group: str, rule_name: str,
                                               log: Log = Log.OFF) -> str:
    """Removes a firewall rule from MySQL Flexible server.

    Args:
        server_name: Name of the server we target.
        resource_group: Azure resource group that the server belongs to.
        rule_name: Name of the rule to be created.
        log: Log level from OFF, VERBOSE or DEBUG.

    Returns:
        Operation result
    """

    az_command: str = commands.get("azure_cli_db_mysql_flexible_server_firewall_rule_delete").format(
        server_name= server_name,
        resource_group=resource_group,
        rule_name=re.sub(rule_name_fix_regex, "-", rule_name),
        log="" if log == Log.OFF else f"--{log.name.lower()}"
    )
    logging.info(literals.get("azure_cli_executing_command").format(command=az_command))

    result: str = cli.call_subprocess_with_result(az_command)
    logging.info(literals.get("azure_cli_command_output").format(output=result))

    return result


if __name__ == "__main__":
    help(__name__)



""" This script allows the user to configure some initial settings """
import argparse
import json
import pkg_resources
import logging

logging.basicConfig(level=logging.INFO)
logging.Formatter("%(asctime)s %(levelname)-8s %(module)-15s %(message)s")
logger = logging.getLogger(__name__)


def main(devops_platform: str, language: str):
    """ Sets the configuration inside settings.json """
    settings_path = pkg_resources.resource_filename("devops_toolset.core", "settings.json")

    with open(settings_path, 'r') as settings_file:
        settings = json.load(settings_file)
        logger.info(f"Retrieved settings.json from {settings_path}")

    settings['platform'] = devops_platform
    settings['language'] = language

    with open(settings_path, 'w') as settings_file:
        logger.info(f"Setting 'devops_platform' -> {devops_platform}")
        logger.info(f"Setting 'language' -> {language}")
        json.dump(settings, settings_file)
        logger.info("Settings successfully saved")

# TODO(alberto.carbonell) Make a guided configuration using clint when no arguments
# TODO(alberto.carbonell) Create argument validators based on enums to avoid input errors


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--devops-platform", default="azuredevops")
    parser.add_argument("--language", default="en")
    args = parser.parse_args()
    main(args.devops_platform, args.language)



"""Base class for ValueDicts"""

import inspect
from typing import List


class ValueDictsBase(object):
    """Base class for ValueDicts"""

    def __init__(self, value_list: List = None):
        """Plains all values from all the dictionaries passed as a parameter
        plus the ones defined in the instanced class.

        Args:
            value_list: List of class types where values should be found.
        """

        self._all_dictionaries = self.get_dicts()

        if value_list:
            for values in value_list:
                _external = values()
                self._all_dictionaries += _external.get_dicts()

        self.all = {}
        for dictionary in self._all_dictionaries:
            self.all.update(dictionary[1])

    def get(self, key: str) -> str:
        """Gets a literal from a given key.

        Args:
            key: used for getting the value
        """

        return str(self.all[key])

    def get_dicts(self):
        """Gets all dict objects of the class."""

        return inspect.getmembers(self, lambda m: type(m) is dict and len(m) > 0)



"""sonarcloud.io / SonarQube tools"""

from devops_toolset.core.app import App
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.devops_platforms.Literals import Literals as DevopsLiterals
from devops_toolset.tools.xcoding64 import encode
from devops_toolset.devops_platforms.constants import Urls
from devops_toolset.tools.git import simplify_branch_name
import configparser
import devops_toolset.core.log_tools as log_tools
import logging
import requests

app: App = App()
literals = LiteralsCore([DevopsLiterals])
platform_specific = app.load_platform_specific("environment")


def get_quality_gate_status(properties_file_path: str, token: str, branch: str = None, pull_request: bool = False):
    """Gets the status of the quality gate of a project

    Args:
        properties_file_path: Path to sonar-project.properties file. Keys
            sonar.projectKey and sonar.organization are mandatory.
        token: SonarCloud token used for authentication
        branch: Git branch name to get the quality gate for. If None, master
            will be assumed.
        pull_request: True if the analysis was originated by a pull request.
    """

    log_tools.log_indented_list(literals.get("function_params"),
                                log_tools.get_parameter_value_list(locals()),
                                log_tools.LogLevel.debug)

    logging.info(literals.get("sonar_getting_qg").format(branch=branch))
    logging.info(literals.get("sonar_pr_mode").format(pull_request=pull_request))

    logging.info(literals.get("sonar_config_file").format(file=properties_file_path))
    sonar_url, sonar_project_key, sonar_organization = read_sonar_properties_file(properties_file_path)

    get_project_quality_gate_status(sonar_url, sonar_project_key, token, branch, pull_request)


def get_project_quality_gate_status(
        url: str, project_key: str, token: str, branch: str = None, pull_request: bool = False):
    """Gets the status of the quality gate of a project

    Args:
        url: Url base for the SonarX service.
        project_key: SonarX project key to get quality gate from.
        token: SonarX token used for authentication.
        branch: Git branch name to get the quality gate for. If None, master
            will be assumed.
        pull_request: True if the analysis was originated by a pull request.
    """

    log_tools.log_indented_list(literals.get("function_params"),
                                log_tools.get_parameter_value_list(locals()),
                                log_tools.LogLevel.debug)

    token_base64 = encode(f"{token}:")
    basic_auth_token = f"Basic {token_base64}"
    headers = {"Authorization": basic_auth_token}
    branch_segment = generate_branch_segment(branch, pull_request)

    url = f"{url}{Urls.SONAR_QUALITY_GATE_PARTIAL_URL}{project_key}{branch_segment}"
    logging.info(literals.get("sonar_qg_url").format(url=url))

    response = requests.get(url, headers=headers)
    if response.status_code != 200:
        raise ValueError(literals.get("sonar_unexpected_status_code").format(statusCode=response.status_code))

    quality_gate_data = response.json()
    logging.info(literals.get("sonar_qg_json").format(json=quality_gate_data))

    if quality_gate_data["projectStatus"]["status"] == "OK":
        logging.info(literals.get("sonar_qg_ok"))
    else:
        for condition in quality_gate_data["projectStatus"]["conditions"]:
            if condition["status"] == "ERROR":
                logging.error(literals.get("sonar_invalid_metric").format(
                    metricKey=condition["metricKey"],
                    actualValue=condition["actualValue"],
                    comparator=condition["comparator"],
                    errorThreshold=condition["errorThreshold"]))

        platform_specific.end_task(platform_specific.ResultType.fail)


def read_sonar_properties_file(path: str):
    """Reads a sonar-project.properties file

    It adds a temporary dummy section to be correctly parsed by configParser

    Args:
        path: Path to the sonar-project.properties file

    Returns:
        Tuple containing url, projectKey, organization
    """

    with open(path, "r") as file:
        data = f"[temp]\n{file.read()}"

    properties_file_parser = configparser.ConfigParser()
    properties_file_parser.read_string(data)

    sonar_url = properties_file_parser.get("temp", "sonar.host.url")
    sonar_project_key = properties_file_parser.get("temp", "sonar.projectKey")
    sonar_organization = properties_file_parser.get("temp", "sonar.organization")

    return sonar_url, sonar_project_key, sonar_organization


def generate_branch_segment(branch: str = "master", pull_request: bool = False):
    """Generates the branch segment or the URL used to request the quality gate
    status

    Args:
        branch: Git branch. If None, master will be assumed.
        pull_request: True if the analysis was originated by a pull request
    """

    branch = simplify_branch_name(branch)

    if pull_request:
        branch = branch.replace("pull/", "")
        return f"&pullRequest={branch}"
    else:
        return f"&branch={branch}"


if __name__ == "__main__":
    help(__name__)



""" Contains dotnet utilities """

import devops_toolset.tools.cli
from devops_toolset.core.app import App
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.project_types.dotnet.Literals import Literals as DotnetLiterals
from devops_toolset.core.CommandsCore import CommandsCore
from devops_toolset.project_types.dotnet.commands import Commands as DotnetCommands

app: App = App()
literals = LiteralsCore([DotnetLiterals])
commands = CommandsCore([DotnetCommands])


def convert_debug_parameter(value: bool) -> str:
    """ Converts force boolean into the correspondent parameter
    Arguments:
        value: The value to be converted

    Returns: --force when true, empty value otherwise
    """
    if value:
        return "--verbosity=diagnostic"
    return ""


def convert_force_parameter(value: bool) -> str:
    """ Converts force boolean into the correspondent parameter
    Arguments:
        value: The value to be converted

    Returns: --force when true, empty value otherwise
    """
    if value:
        return "--force"
    return ""


def convert_with_restore_parameter(value: bool) -> str:
    """ Converts with_restore boolean into the correspondent parameter
    Arguments:
        value: The value to be converted

    Returns: --no-restore when false, empty value otherwise
    """
    if not value:
        return "--no-restore"
    return ""


def restore(path: str, force: bool = False, debug: bool = False):
    """ Performs a dotnet restore in the desired path
    Arguments:
        path: The path where restore will be executed.
        force: Adds --force argument.
        debug: Enables diagnostic logs to the command.

    More info: https://docs.microsoft.com/es-es/dotnet/core/tools/dotnet-restore

    """
    devops_toolset.tools.cli.call_subprocess(commands.get("dotnet_restore").format(
        force=convert_force_parameter(force),
        path=path,
        debug=convert_debug_parameter(debug)),
        log_before_process=[literals.get("dotnet_restore_before").format(path=path)],
        log_after_err=[literals.get("dotnet_restore_err").format(path=path)])


def build(path: str, configuration: str = "Release", output: str = ".", framework: str = "net5.0",
          runtime: str = "linux-x64", with_restore: bool = False, force: bool = False, debug: bool = False):
    """ Performs a dotnet build in the desired path
    Arguments:
        path: The path where build will be executed.
        configuration: The configuration used for build. Default is "Release".
        output: Adds --output argument. Specifies the output path of the build command. Defaults "."
        framework: The dotnet framework used to build. Default is "net5.0".
        runtime: The runtime used to build. Default is "linux-x64".
        with_restore: Adds --no-restore argument when False. Default to False.
        force: Adds --force argument.
        debug: Enables diagnostic logs to the command.

    More info: https://docs.microsoft.com/es-es/dotnet/core/tools/dotnet-build

    """
    devops_toolset.tools.cli.call_subprocess(commands.get("dotnet_build").format(
        force=convert_force_parameter(force),
        path=path,
        debug=convert_debug_parameter(debug),
        configuration=configuration,
        output=output,
        framework=framework,
        runtime=runtime,
        with_restore=convert_with_restore_parameter(with_restore)),
        log_before_process=[literals.get("dotnet_build_before").format(path=path)],
        log_after_err=[literals.get("dotnet_build_err").format(path=path)])


if __name__ == "__main__":
    help(__name__)



"""devops_platforms module literals."""

from devops_toolset.core.ValueDictsBase import ValueDictsBase
from devops_toolset.core.app import App

app: App = App()


class Literals(ValueDictsBase):
    """ValueDicts for the devops_platforms module."""

    # Add your core literal dictionaries here
    _info = {
        "environment_variable_log": _("=> {key} {value}"),
        "sonar_getting_qg": _("Getting quality gate for branch {branch} (original name)."),
        "sonar_pr_mode": _("Pull request mode: {pull_request}"),
        "sonar_config_file": _("Using {file} as the Sonar* configuration file."),
        "sonar_qg_ok": _("Quality gate succeeded"),
        "sonar_qg_json": _("This is the JSON data returned by Sonar*:\n\n{json}"),
        "sonar_qg_url": _("Request URL:\n{url}")
    }
    _errors = {
        "sonar_invalid_metric": _("Invalid metric value for {metricKey}: {actualValue} {comparator} {errorThreshold}"),
        "sonar_unexpected_status_code":
            _("Status code got from Sonar* was not 200, but {statusCode}. Please check it out."),
    }



"""i18n module literals"""

from devops_toolset.core.ValueDictsBase import ValueDictsBase
from devops_toolset.core.app import App

app: App = App()


class Literals(ValueDictsBase):
    """ValueDicts for the wordpress module."""

    _titles = {}
    _info = {
        "i18n_mo_file_removed": _("[POT file compilation] File removed: {path}"),
        "i18n_pot_file_path": _("[POT file generation] File path: {path}"),
        "i18n_po_file_list": _("[POT file compilation] Python file list to be compiled:"),
        "i18n_pot_file_list": _("[POT file generation] Python file list to be parsed:"),
        "i18n_pot_file_removed": _("[POT file generation] File removed: {path}"),
        "i18n_pot_script": _("[POT file generation] Script being used: {script}"),
        "i18n_po_command": _("[POT file compilation] Command being called: {command}"),
        "i18n_pot_command": _("[POT file generation] Command being called: {command}"),
    }
    _errors = {}



"""Helper functions for Git-related task automation."""

# ! python

import devops_toolset.core.app
import devops_toolset.filesystem.paths
import devops_toolset.filesystem.paths as path_tools
import devops_toolset.tools.cli
import logging
import os
import pathlib
import re

from clint.textui import prompt
from devops_toolset.core.CommandsCore import CommandsCore
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.filesystem.constants import FileNames, Directions
from devops_toolset.tools.commands import Commands as ToolsCommands
from devops_toolset.tools.Literals import Literals as ToolsLiterals

app: devops_toolset.core.app.App = devops_toolset.core.app.App()
literals = LiteralsCore([ToolsLiterals])
commands = CommandsCore([ToolsCommands])
platform_specific = app.load_platform_specific("environment")


def add_gitignore_exclusion(path: str, exclusion: str):
    """Adds an exclusion in a .gitignore file.

    It adds the exclusion at the end of the file.

    Args:
        path: Path to the .gitignore file.
        exclusion: Exclusion to be added (whole line must be passed).
    """

    with open(path, "a") as _gitignore:
        _gitignore.write(f"\n{exclusion}\n")


def find_gitignore_exclusion(path: str, exclusion: str) -> bool:
    """Finds if an excusion exisits in a .gitignore file.

    It tries to find the exclusion.

    Args:
        path: Path to the .gitignore file.
        exclusion: Exclusion to find (whole line must be passed not including
            newline).

    Returns:
        True if the exclusion is found, False otherwise.
    """

    with open(path, "r") as _gitignore:
        index = _gitignore.read().find(f"{exclusion}\n")

    return index > -1


def get_gitignore_path(path: str = None, direction: Directions = Directions.ASCENDING) -> str:
    """Gets the path to the .gitignore file.

    If no path parameter is passed to the function it returns the path to the
    .gitignore file in the project root.
    If a path is passed it returns the path to the .gitignore file that is
    closer to the given path in the specified direction.

    Args:
        path: Optional path to start looking for a .gitignore file.
        direction: The direction of the seek (ascending by default).

    Returns:
        The path to the closest/root .gitignore file
    """
    if path is None:
        gitignore_path = pathlib.Path.joinpath(
            pathlib.Path(devops_toolset.filesystem.paths.get_project_root()), FileNames.GITIGNORE_FILE)
        if not pathlib.Path(gitignore_path).exists():
            raise FileNotFoundError
        return gitignore_path

    return str(devops_toolset.filesystem.paths.get_filepath_in_tree(FileNames.GITIGNORE_FILE, direction))


def git_commit(skip: bool):
    """Add modified files to .git repository and commit

    Args:
        skip: Boolean that determines if the user want or don't want create the .git repository.
    """
    if not skip:
        devops_toolset.tools.cli.call_subprocess(commands.get("git_add"),
                                                 log_before_process=[literals.get(
                                                     "git_before_adding_project_structure_files_to_stage")],
                                                 log_after_err=[
                                                     literals.get("git_err_adding_project_structure_files_to_stage")])
        devops_toolset.tools.cli.call_subprocess(commands.get("git_commit_m").format(
            message=literals.get("git_add_project_structure_message")),
            log_before_process=[literals.get("git_before_project_structure_commit")],
            log_after_err=[literals.get("git_err_commit_project_structure")],
            log_after_out=[literals.get("git_after_project_structure_commit")])


def git_init(path: str, skip: bool):
    """Initialize .git repository

    Args:
        path: Path where it creates the .git repository.
        skip: Boolean that determines if the user want or don't want create the .git repository.
    """

    if not skip:
        init_git = prompt.yn(literals.get("git_init_repo"))
        if init_git:
            devops_toolset.tools.cli.call_subprocess(commands.get("git_init").format(path=path),
                                                     log_before_process=[literals.get("git_repo_to_be_created")],
                                                     log_after_err=[literals.get("git_err_create_repo")],
                                                     log_after_out=[literals.get("git_repo_created")])


def git_tag_add(tag_name: str, commit_name: str, push_to_origin: bool = True):
    """ Adds a tag to a git commit

     Args:
        tag_name: Name of the tag to be added
        commit_name: Name of the commit.
            Git will need the checksum name, or part of it.
            F.I: If the commit name is 9fceb02d0ae598e95dc970b74767f19372d61af8, the checksum will be 9fceb02
        push_to_origin: If True, will push the tag to origin

    """
    # TODO(alberto.carbonell): Check if tag_name is correct (not duplicated, semver compliant, etc.)
    # TODO(alberto.carbonell): Check if commit exists in current branch
    devops_toolset.tools.cli.call_subprocess(
        commands.get("git_tag_add").format(tag_name=tag_name, commit_name=commit_name),
        log_before_process=[literals.get("git_tag_add_init").format(tag_name=tag_name, commit_name=commit_name)],
        log_after_err=[literals.get("git_tag_add_err")])

    if push_to_origin:
        devops_toolset.tools.cli.call_subprocess(commands.get("git_push_tag").format(tag_name=tag_name),
                                                 log_before_process=[
                                                     literals.get("git_push_tag_init").format(tag_name=tag_name)],
                                                 log_after_err=[literals.get("git_push_tag_err")])


def git_tag_delete(tag_name: str, push_to_origin: bool = True):
    """ Deletes a tag from git
    Args:
        tag_name: Name of the tag to be deleted
        push_to_origin: If True, will push the tag deletion to origin
    """
    devops_toolset.tools.cli.call_subprocess(commands.get("git_tag_delete").format(tag_name=tag_name),
                                             log_before_process=
                                             [literals.get("git_tag_delete_init").format(tag_name=tag_name)],
                                             log_after_err=
                                             [literals.get("git_tag_delete_err").format(tag_name=tag_name)])
    if push_to_origin:
        devops_toolset.tools.cli.call_subprocess(commands.get("git_push_tag_delete").format(tag_name=tag_name),
                                                 log_before_process=[literals.get("git_push_tag_delete_init")
                                                 .format(tag_name=tag_name)],
                                                 log_after_err=[literals.get("git_push_tag_delete_err")])


def purge_gitkeep(path: str = None):
    """Deletes .gitkeep file if exists and there are more files in the path."""

    if not devops_toolset.filesystem.paths.is_valid_path(path, True):
        raise ValueError(literals.get("git_non_valid_dir_path"))

    path_object = pathlib.Path(path)
    guess_gitkeep_file = pathlib.Path(pathlib.Path.joinpath(path_object, ".gitkeep"))
    if not path_tools.is_empty_dir(path) and guess_gitkeep_file.exists():
        logging.info(literals.get("git_purging_gitkeep").format(path=path))
        os.remove(guess_gitkeep_file)


def get_current_branch_simplified(branch: str, environment_variable_name: str = "DT_CURRENT_BRANCH_SIMPLIFIED") -> str:
    """Creates an environment variable from a branch name (simplified)

    Args:
        branch: Git branch name to be simplified and stored in an environment
            variable
        environment_variable_name: Name of the environment variable to be
            created. Defaults to "DT_CURRENT_BRANCH_SIMPLIFIED".

    Returns:
        Branch name simplified.
    """

    simplified_branch_name = simplify_branch_name(branch)
    platform_specific.create_environment_variables({environment_variable_name: simplified_branch_name})

    return simplified_branch_name


def simplify_branch_name(branch: str):
    """Simplifies a branch name.

    e.g:
        refs/heads/master => master
        refs/heads/feature/name => feature/name
        refs/pull/1/merge => pull/1

    If none of these cases the original branch name is returned.

    Args:
        branch: Long name of the branch to be simplified
    """

    if branch.startswith("refs/heads/"):
        return branch.replace("refs/heads/", "")

    if branch.startswith("refs/pull/"):
        return branch.replace("refs/", "").replace("/merge", "")

    return branch


def update_gitignore_exclusion(path: str, regex: str, value: str):
    """Updates an existing excusion in a .gitignore file.

    It updates the exclusion in the .gitignore file replacing the passed value using the RegEx.

    Args:
        path: Path to the .gitignore file.
        regex: RegEx with only 1 capturing group, whose content will be replaced.
        value: String that will replace the RegEx capture group.
    Raises:
        ValueError: When regex has no capture groups or more than one
    """

    compiled_regex = re.compile(regex)

    if compiled_regex.groups != 1:
        raise ValueError(literals.get("git_regex1cg"))

    with open(path, "r+") as _gitignore:
        content = _gitignore.read()

    from_index = 0
    iterations = len(compiled_regex.findall(content))
    while iterations > 0:
        match = compiled_regex.search(content, from_index)
        content = content[:match.regs[1][0]] + value + content[match.regs[1][1]:]
        from_index = match.regs[1][1]
        iterations -= 1

    with open(path, "w") as _gitignore:
        _gitignore.write(content)


if __name__ == "__main__":
    help(__name__)



"""Defines constants for the module."""

required_files_suffixes = {
    "site_configuration_file_path": "site.json"
}

theme_metadata_parse_regex = ": (.+)"
functions_php_mytheme_regex = "(mytheme)(?=_[\w\d\sáéíóú'-.])"

wordpress_constants_json_resource = \
    "https://raw.githubusercontent.com/aheadlabs/devops-toolset/master/src/" \
    "devops_toolset/project_types/wordpress/wordpress-constants.json"



"""Contains several tools for WordPress"""
import json
import logging
import os
import pathlib
import stat
import sys
import devops_toolset.tools.dicts
from typing import List, Tuple

import requests

import devops_toolset.core.log_tools
import devops_toolset.filesystem.paths as paths
import devops_toolset.filesystem.tools
import devops_toolset.project_types.wordpress.constants as wp_constants
import devops_toolset.project_types.wordpress.wp_cli as wp_cli
import devops_toolset.tools.git as git_tools
from devops_toolset.core.CommandsCore import CommandsCore
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.core.app import App
from devops_toolset.devops_platforms import constants as devops_platforms_constants
from devops_toolset.devops_platforms.azuredevops.Literals import Literals as PlatformLiterals
from devops_toolset.project_types.wordpress.Literals import Literals as WordpressLiterals
from devops_toolset.project_types.wordpress.basic_structure_starter import BasicStructureStarter
from devops_toolset.project_types.wordpress.commands import Commands as WordpressCommands

import re

app: App = App()
platform_specific_restapi = app.load_platform_specific("restapi")
literals = LiteralsCore([WordpressLiterals])
platform_literals = LiteralsCore([PlatformLiterals])
commands = CommandsCore([WordpressCommands])


def add_wp_options(wp_options: dict, wordpress_path: str, debug: bool = False):
    """Adds or updates WordPress options in the wp_options table

    Args:
        wp_options: WordPress options.
        wordpress_path: Path to the WordPress installation.
        debug: If True logs debug information.
    """
    for option in wp_options:
        wp_cli.add_update_option(option, wordpress_path, debug)


def convert_wp_config_token(token: str, wordpress_path: str) -> str:
    """ Replaces [] tokens inside configuration parameters using php syntax

    Args:
        token: The token to replace (for example: [date|Y.m.d-Hisve])
        wordpress_path: Wordpress installation path
    """
    result = token
    # parse token [date|Y.m.d-Hisve]
    if token.find("[date|") != -1:
        date_format = token[token.find("[date|") + 1:token.find("]")]
        date_token = date_format.split("|")[1]
        result = result.replace(
            "[" + date_format + "]", wp_cli.eval_code("echo date('" + date_token + "');", wordpress_path))
    # NOTE: Add more tokens if needed
    return result


def create_wp_cli_bat_file(phar_path: str):
    """Creates a .bat file for WP-CLI.

    Args:
        phar_path: Path to the .phar file.
    """

    path = pathlib.Path(phar_path)
    bat_path = pathlib.Path.joinpath(path.parent, "wp.bat")

    with open(bat_path, "w") as bat:
        bat.write("@ECHO OFF\n")
        bat.write(f"php \"{phar_path}\" %*")


def create_configuration_file(environment_configuration: dict, wordpress_path: str, database_user_password: str):
    """Creates the wp-config-php WordPress configuration file using WP-CLI.

    All parameters are obtained from a site configuration file.

    For more information see:
        https://developer.wordpress.org/cli/commands/config/create/

    Args:
        environment_configuration: Parsed environment configuration.
        wordpress_path: Path to WordPress files.
        database_user_password: Password for the database user configured in at the wp-config.php file.
    """

    wp_cli.create_configuration_file(wordpress_path=wordpress_path,
                                     db_host=environment_configuration["database"]["host"],
                                     db_name=environment_configuration["database"]["db_name"],
                                     db_user=environment_configuration["database"]["db_user"],
                                     db_pass=database_user_password,
                                     db_prefix=environment_configuration["database"]["table_prefix"],
                                     db_charset=environment_configuration["database"]["charset"],
                                     db_collate=environment_configuration["database"]["collate"],
                                     skip_check=environment_configuration["database"]["skip_check"],
                                     debug=environment_configuration["wp_cli_debug"]
                                     )


def create_users(users: dict, wordpress_path: str, debug: bool):
    """Creates WordPress users.

    Args:
        users: Users based on #/definitions/user at
            https://dev.aheadlabs.com/schemas/json/wordpress-site-schema.json
        wordpress_path: Path to WordPress files.
        debug: If present, --debug will be added to the command showing all debug trace information.
    """

    for user in users:
        # Create the user if does not exist
        if not wp_cli.user_exists(user["user_login"], wordpress_path, debug):
            wp_cli.create_user(user, wordpress_path, debug)

        # Warn the user I am skipping the creation
        else:
            logging.warning(literals.get("wp_wpcli_user_exists").format(user=user["user_login"]))


def download_wordpress(site_configuration: dict, destination_path: str, wp_cli_debug: bool = False):
    """ Downloads the latest version of the WordPress core files using a site configuration file.

    For more information see:
        https://developer.wordpress.org/cli/commands/core/download/

    Args:
        site_configuration: parsed site configuration.
        destination_path: Path where WP-CLI will be downloaded.
        wp_cli_debug: True if logging must be verbose.
    """

    if not paths.is_valid_path(destination_path):
        raise ValueError(literals.get("wp_non_valid_dir_path"))

    version = site_configuration["settings"]["version"]
    locale = site_configuration["settings"]["locale"]
    skip_content = site_configuration["settings"]["skip_content_download"]
    wp_cli.download_wordpress(destination_path, version, locale, skip_content, wp_cli_debug)
    git_tools.purge_gitkeep(destination_path)


def download_wordpress_plugin(plugin_config: dict, destination_path: str):
    """Downloads a WordPress plugin from an URL.

    NOTE: The URL must download a zip file that contains the plugin. If the ZIP
        contains a non-standard inner structure, the calling process will
        produce side-effects.

    Args:
        plugin_config: Plugin configuration.
        destination_path: Path where the plugin will be downloaded.
    """
    with open(destination_path, "wb") as file:
        response = requests.get(plugin_config["source"])
        file.write(response.content)


def export_database(environment_config: dict, wordpress_path: str, dump_file_path: str):
    """Exports a WordPress database to a dump file using a site configuration file.

    All parameters are obtained from a site configuration file.

    For more information see:
        https://developer.wordpress.org/cli/commands/db/export/

    Args:
        environment_config: parsed site configuration.
        wordpress_path: Path to WordPress files.
        dump_file_path: Path to the destination dump file.
    """
    wp_cli.export_database(wordpress_path, dump_file_path, environment_config["wp_cli_debug"])


def get_constants() -> dict:
    """Gets all the constants from a WordPress constants resource.

    For more information see:
        https://dev.aheadlabs.com/schemas/json/wordpress-constants-schema.json


    Returns:
        All the constants in a dict object.
    """

    response = requests.get(wp_constants.wordpress_constants_json_resource)
    data = json.loads(response.content)

    return data


def get_project_structure(url_resource: str) -> dict:
    """Gets the project structure from a WordPress project structure file located on an url resource.

    For more information see:
        https://dev.aheadlabs.com/schemas/json/project-structure-schema.json

    Args:
        url_resource: Full url resource to the WordPress project structure file.

    Returns:
        Project structure in a dict object.
    """
    request = requests.get(url_resource)
    return request.json()


def get_required_file_paths(path: str, required_file_patterns: List[str]) -> Tuple:
    """Returns file paths in a tuple from the file name patterns.

    Args:
        path: Where to look for the files.
        required_file_patterns: glob patterns of the file names to be found.

    Returns:
        Tuple with the file paths in the following order:
        - site configuration JSON file
        - site environments JSON file
        - project structure JSON file
    """

    # required_file_patterns = wordpress.constants.required_files_suffixes
    #
    # for required_file_pattern in required_file_patterns:
    #     if required_file_pattern.endswith()

    result = []
    for required_file_pattern in required_file_patterns:
        result.append(paths.get_file_path_from_pattern(path, required_file_pattern))

    if len(result) == 0:
        logging.info(literals.get("wp_required_file_paths_not_found"))
    else:
        devops_toolset.core.log_tools.log_indented_list(literals.get("wp_required_file_paths_found"),
                                         result, devops_toolset.core.log_tools.LogLevel.info)

    return tuple(result)


def get_site_configuration(path: str) -> dict:
    """Gets the WordPress site configuration from a site configuration file.

    For more information see:
        https://dev.aheadlabs.com/schemas/json/wordpress-site-schema.json

    Args:
        path: Full path to the WordPress project structure file.

    Returns:
        Site configuration in a dict object.
    """

    with open(path, "r", encoding="utf-8") as config_file:
        data = config_file.read()
        return json.loads(data)


def get_environment(site_config: dict, environment_name: str) -> dict:
    """Gets the environment that matches the name passed as a parameter.

    Args:
        site_config: Site configuration.
        environment_name: Name of the environment to be retrieved.
    """

    filtered = filter(lambda environment_x: environment_x['name'] == environment_name, site_config['environments'])
    environment_list = list(filtered)

    if len(environment_list) == 0:
        raise ValueError(literals.get('wp_env_x_not_found').format(environment=environment_name))

    if len(environment_list) > 1:
        logging.warning(literals.get('wp_environment_x_found_multiple').format(environment=environment_name))

    environment = environment_list[0]

    # Update URL constants (with the _url suffix) prepending the base URL
    url_keys = devops_toolset.tools.dicts.filter_keys(environment["wp_config"], "_url$")
    for key in url_keys:
        environment["wp_config"][key]["value"] = environment["base_url"] + environment["wp_config"][key]["value"]

    return environment


def get_wordpress_path_from_root_path(root_path: str, constants: dict = None) -> str:
    """ Gets the wordpress path based on the constants.json from a desired root path

    Args:
        root_path: Full path of the project.
        constants: WordPress constants.
    """
    logging.info(literals.get("wp_root_path").format(path=root_path))

    # Get constants if not passed
    if constants is None:
        constants = get_constants()

    # Get wordpress path from the constants
    wordpress_relative_path = constants["paths"]["wordpress"]
    wordpress_path = pathlib.Path.joinpath(pathlib.Path(root_path), wordpress_relative_path).as_posix()
    logging.info(literals.get("wp_wordpress_path").format(path=wordpress_path))

    return wordpress_path


def import_content_from_configuration_file(site_configuration: dict, environment_config: dict,
                                           root_path: str, global_constants: dict):
    """ Imports WordPress posts content specified on a site_configuration file.
    NOTE: content entries in the configuration file must be named after post
    types in singular form. Otherwise they will be ignored. ie: post, page.

    Args:
        site_configuration: Parsed site configuration.
        environment_config: Parsed environment configuration.
        root_path: Path to the root repository.
        global_constants: Parsed global constants.
    """
    # If no content to import, then do nothing
    if "content" not in site_configuration:
        return

    # Get paths and parameters
    wxr_path = pathlib.Path.joinpath(pathlib.Path(root_path), global_constants["paths"]["content"]["wxr"])
    wordpress_path = str(pathlib.Path.joinpath(pathlib.Path(root_path), global_constants["paths"]["wordpress"]))
    author_handling = site_configuration["content"]["author_handling"]

    if author_handling == "mapping.csv":
        authors_path = str(pathlib.Path.joinpath(wxr_path, "mapping.csv"))
        authors = authors_path if not devops_toolset.filesystem.tools.is_file_empty(authors_path) else "skip"
    else:
        authors = author_handling

    debug_info = environment_config["wp_cli_debug"]

    for content_type in site_configuration["content"]["sources"]:

        # File name will be the {wxr_path}/{content_type}.xml
        content_path = str(pathlib.Path.joinpath(wxr_path, f"{content_type}.xml"))

        # Delete content before importing (to avoid duplicating content)
        wp_cli.delete_post_type_content(wordpress_path, content_type, debug_info)

        # Import new content
        wp_cli.import_wxr_content(wordpress_path, content_path, authors, debug_info)


def install_plugins_from_configuration_file(site_configuration: dict, environment_config: dict, global_constants: dict,
                                            root_path: str, skip_partial_dumps: bool):
    """Installs WordPress's plugin files using WP-CLI.

       For more information see:
           https://developer.wordpress.org/cli/commands/plugin/install/

       Args:
           site_configuration: Parsed site configuration.
           environment_config: Parsed environment configuration.
           global_constants: Parsed global constants.
           root_path: Path to project root.
           skip_partial_dumps: If True skips database dumps.
       """
    # Get data needed in the process
    plugins: dict = site_configuration["settings"]["plugins"]
    root_path_obj = pathlib.Path(root_path)
    wordpress_path = str(pathlib.Path.joinpath(root_path_obj, global_constants["paths"]["wordpress"]))
    plugins_path = str(pathlib.Path.joinpath(root_path_obj, global_constants["paths"]["content"]["plugins"]))
    debug_info = environment_config["wp_cli_debug"]

    for plugin in plugins:
        # Get plugin path
        plugin_path = paths.get_file_path_from_pattern(plugins_path, f"{plugin['name']}*.zip")
        logging.info(literals.get("wp_plugin_path").format(path=plugin_path))

        # Download plugin if needed
        if plugin["source_type"] == "url":
            download_wordpress_plugin(plugin, plugin_path)

            # Once downloaded, should have a .zip under plugins path, so can freely add this source as a .zip one for
            # further installing this plugin as a zip
            plugin["source_type"] = "zip"

        if plugin["source_type"] == "zip":
            plugin["source"] = plugin_path

        wp_cli.install_plugin(plugin["name"], wordpress_path, plugin["activate"], plugin["force"], plugin["source"],
                              debug_info)

        # Backup database after plugin install
        if not skip_partial_dumps:
            database_path = pathlib.Path.joinpath(root_path_obj, global_constants["paths"]["database"])
            core_dump_path_converted = convert_wp_config_token(
                site_configuration["settings"]["dumps"]["plugins"], wordpress_path)
            database_core_dump_path = pathlib.Path.joinpath(database_path, core_dump_path_converted)
            export_database(site_configuration, wordpress_path, database_core_dump_path.as_posix())

        # Warn the user we are skipping the backup dump
        else:
            logging.warning(literals.get("wp_wpcli_export_db_skipping_as_set").format(dump="plugins"))


def install_recommended_plugins():
    """ Uses TGMPA core to decide and install automatically the recommended plugins.

    See Also: https://tgmpluginactivation.com/
    See Also: https://github.com/itspriddle/wp-cli-tgmpa-plugin
    Args:

    """
    # TODO(alberto.carbonell) Develop an WP-cli extension.
    pass


def install_wordpress_core(site_config: dict, environment_config: dict, wordpress_path: str, admin_password: str):
    """Installs WordPress core files using a site configuration file.

        For more information see:
            https://developer.wordpress.org/cli/commands/core/install/

        Args:
            site_config: Parsed site configuration.
            environment_config: Parsed environment configuration.
            wordpress_path: Path to WordPress files.
            admin_password: Password for the WordPress administrator user
        """

    # Set/expand variables before using WP CLI
    admin_user = site_config["settings"]["wp_admin"]["user"]
    admin_email = site_config["settings"]["wp_admin"]["email"]
    skip_email = site_config["settings"]["wp_admin"]["skip_email"]
    title = site_config["settings"]["title"]
    debug_info = environment_config["wp_cli_debug"]
    url = environment_config["wp_config"]["site_url"]["value"]
    wp_cli.install_wordpress_core(wordpress_path, url, title, admin_user, admin_email, admin_password,
                                  skip_email, debug_info)


def install_wp_cli(install_path: str = "/usr/local/bin/wp"):
    """Downloads and installs the latest version of WP-CLI.

    For more information see:
        https://make.wordpress.org/cli/handbook/installing/

    Args:
        install_path: Path where WP-CLI will be installed. It must be in the
            PATH/BIN of the operating system.
    """

    wp_cli_phar = "wp-cli.phar"
    wp_cli_download_url = f"https://raw.githubusercontent.com/wp-cli/builds/gh-pages/phar/{wp_cli_phar}"
    install_path = pathlib.Path(install_path)
    file_path = str(pathlib.Path.joinpath(install_path, wp_cli_phar))

    if not pathlib.Path.is_dir(install_path):
        raise ValueError(literals.get("wp_not_dir"))

    logging.info(literals.get("wp_wpcli_downloading").format(url=wp_cli_download_url))
    response = requests.get(wp_cli_download_url)

    with open(file_path, "wb") as cli:
        cli.write(response.content)

    file_stat = os.stat(file_path)
    os.chmod(file_path, file_stat.st_mode | stat.S_IEXEC)

    if sys.platform == "win32":
        create_wp_cli_bat_file(file_path)

    wp_cli.wp_cli_info()


def install_wordpress_site(site_configuration: dict, environment_config: dict, global_constants: dict,
                           root_path: str, admin_password: str, skip_partial_dumps: bool = False):
    """Installs WordPress core files using WP-CLI.

    This operation requires cleaning the db and doing a backup after the process.

    For more information see:
        https://developer.wordpress.org/cli/commands/core/install/

    Args:
        site_configuration: Parsed site configuration.
        environment_config: Parsed environment configuration.
        global_constants: Parsed global constants.
        root_path: Path to site installation.
        admin_password: Password for the WordPress administrator user.
        skip_partial_dumps: If True skips database dump.
    """

    database_path = global_constants["paths"]["database"]
    root_path_obj = pathlib.Path(root_path)
    wordpress_path = str(pathlib.Path.joinpath(root_path_obj, global_constants["paths"]["wordpress"]))
    wordpress_path_as_posix = str(pathlib.Path(wordpress_path).as_posix())

    # Install wordpress
    install_wordpress_core(site_configuration, environment_config, wordpress_path_as_posix, admin_password)

    # Update description option
    description = site_configuration["settings"]["description"]
    wp_cli.update_database_option(
        "blogdescription", description, wordpress_path_as_posix, environment_config["wp_cli_debug"])

    # Backup database
    if not skip_partial_dumps:
        core_dump_path_converted = \
            convert_wp_config_token(site_configuration["settings"]["dumps"]["core"], wordpress_path)
        database_core_dump_directory_path = pathlib.Path.joinpath(root_path_obj, database_path)
        database_core_dump_path = pathlib.Path.joinpath(database_core_dump_directory_path, core_dump_path_converted)
        export_database(environment_config, wordpress_path_as_posix, database_core_dump_path.as_posix())
        git_tools.purge_gitkeep(database_core_dump_directory_path.as_posix())

    # Warn the user we are skipping the backup dump
    else:
        logging.warning(literals.get("wp_wpcli_export_db_skipping_as_set").format(dump="core"))


def set_wordpress_config_from_configuration_file(environment_config: dict, wordpress_path: str,
                                                 devops_toolset_wordpress_path: str, db_user_password: str) -> None:
    """ Sets all configuration parameters in pristine WordPress core files
    Args:
        environment_config: Environment configuration.
        wordpress_path: Path to wordpress installation.
        devops_toolset_wordpress_path: Path to the source root of the WordPress
            project type in devops-toolset.
        db_user_password: Database user password.

    """

    # Create wp-config.php file
    create_configuration_file(environment_config, wordpress_path, db_user_password)

    # Get config properties to set from site configuration
    wp_config_properties = environment_config["wp_config"]
    debug = environment_config["wp_cli_debug"]

    # Foreach variable to set: execute wp config set
    for prop in wp_config_properties.values():
        # This value will place the value as it gets, without quotes
        value = prop.get("value")
        raw = type(value) != str
        wp_cli.set_configuration_value(
            prop.get("name"), value, prop.get("type"),
            wordpress_path, raw, debug)

    # Add cloudfront snippet to wp_config.php if needed
    add_cloudfront_forwarded_proto_to_config(environment_config, wordpress_path, devops_toolset_wordpress_path)


def add_cloudfront_forwarded_proto_to_config(
        environment_config: dict, wordpress_path: str, devops_toolset_wordpress_path: str):
    """ Adds HTTP_CLOUDFRONT_FORWARDED_PROTO snippet to wp-config.php

    Args:
        environment_config: Environment configuration.
        wordpress_path: Path to wordpress installation.
        devops_toolset_wordpress_path: Path to the source root of the WordPress
            project type in devops-toolset.
    """

    # Exit if there is no True setting for AWS Cloudfront
    if "aws_cloudfront" not in environment_config["settings"] \
            or environment_config["settings"]["aws_cloudfront"] is False:
        return

    file_path = pathlib.Path.joinpath(pathlib.Path(wordpress_path), "wp-config.php")
    if file_path.exists():
        with open(file_path, "r+") as config:
            config_content = config.read()
            pattern = r'/\*\*.*\nrequire_once.*'
            match = re.search(pattern, config_content)
            if match:
                content_new = re.sub(
                    pattern,
                    get_snippet_cloudfront(devops_toolset_wordpress_path) + '\n' + match.group(),
                    config_content)
                config.seek(0)
                config.write(content_new)


def get_snippet_cloudfront(devops_toolset_wordpress_path: str):
    """ Gets HTTP_CLOUDFRONT_FORWARDED_PROTO snippet from a default file.

    Args:
        devops_toolset_wordpress_path: Path to the source root of the WordPress
            project type in devops-toolset.

    Returns:
        HTTP_CLOUDFRONT_FORWARDED_PROTO snippet as a string.
    """

    file_path = pathlib.Path.joinpath(
        pathlib.Path(devops_toolset_wordpress_path), 'default-files/default-cloudfront-forwarded-proto.php')
    if file_path.exists():
        with open(file_path, "r") as snippet_content:
            snippet = snippet_content.read()
            return snippet
    else:
        logging.error(literals.get("wp_file_not_found").format(file=file_path))


def setup_database(environment_config: dict, wordpress_path: str, db_user_password: str, db_admin_password: str = ""):
    """ Uses wp cli create to create a new database from configuration file

    Args:
        environment_config: Parsed environment configuration.
        wordpress_path: Path to WordPress files.
        db_user_password: Password of the database user to be created.
        db_admin_password:  Database administrator user password.
    """

    db_host = environment_config["database"]["host"]
    schema = environment_config["database"]["db_name"]
    db_admin_user = environment_config["database"]["db_admin_user"]
    db_user = environment_config["database"]["db_user"]
    wp_cli_debug = environment_config["wp_cli_debug"]

    wp_cli.create_database(wordpress_path, wp_cli_debug, db_admin_user, db_admin_password, schema)

    wp_cli.create_wordpress_database_user(
        wordpress_path, db_admin_user, db_admin_password, db_user, db_user_password, schema, db_host)


def start_basic_project_structure(root_path: str) -> None:
    """ Creates a basic structure of a wordpress project based on the project-structure.json

    Args:
        root_path: Full path where the structure will be created
    """

    logging.info(literals.get("wp_creating_project_structure"))

    structure_file_path = pathlib.Path.joinpath(pathlib.Path(root_path), "wordpress-project-structure.json")
    # Parse project structure configuration
    if pathlib.Path.exists(structure_file_path):
        project_structure = get_site_configuration(str(structure_file_path))
        logging.info(literals.get("wp_project_structure_creating_from_file").format(file_name=structure_file_path))
    else:
        project_structure = get_project_structure(devops_platforms_constants.Urls.DEFAULT_WORDPRESS_PROJECT_STRUCTURE)
        logging.info(literals.get("wp_project_structure_creating_from_default_file").format(
            resource=devops_platforms_constants.Urls.DEFAULT_WORDPRESS_DEVELOPMENT_THEME_STRUCTURE))

    project_starter = BasicStructureStarter()

    # Iterate through every item recursively
    for item in project_structure["items"]:
        project_starter.add_item(item, root_path)

    logging.info(literals.get("wp_created_project_structure"))


if __name__ == "__main__":
    help(__name__)



"""Contains tools for working with the command line"""

import devops_toolset.core.log_tools
import subprocess
from pyfiglet import Figlet
from typing import List


def print_title(text: str):
    """Prints a title in the console"""
    f = Figlet()
    print(f.renderText(text))


def call_subprocess_with_result(command: str) -> str:
    """Calls a subprocess and returns the stdout

        Args:
            command: Command to be executed.
        """
    process = subprocess.Popen(command.strip(), shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = process.communicate()
    process.wait()

    if out:
        return out.decode("utf-8", errors="backslashreplace")


def call_subprocess(command: str, log_before_process: List[str] = None,
                    log_before_out: List[str] = None, log_after_out: List[str] = None,
                    log_before_err: List[str] = None, log_after_err: List[str] = None):
    """Calls a subprocess.

    Args:
        command: Command to be executed.
        log_before_process: List of strings to log as info before the process
            call.
        log_before_out: List of strings to log as info before the stdout, if
            no errors.
        log_after_out: List of strings to log as info after the stdout, if
            no errors.
        log_before_err: List of strings to log as error before the stderr, if
            errors.
        log_after_err: List of strings to log as error after the stderr, if
            errors.
    """

    devops_toolset.core.log_tools.log_list([command], devops_toolset.core.log_tools.LogLevel.info)
    devops_toolset.core.log_tools.log_list(log_before_process, devops_toolset.core.log_tools.LogLevel.info)

    process = subprocess.Popen(command.strip(), shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = process.communicate()
    process.wait()

    if out:
        devops_toolset.core.log_tools.log_list(log_before_out, devops_toolset.core.log_tools.LogLevel.info)
        devops_toolset.core.log_tools.log_stdouterr(out, devops_toolset.core.log_tools.LogLevel.info)
        devops_toolset.core.log_tools.log_list(log_after_out, devops_toolset.core.log_tools.LogLevel.info)

    if err and process.returncode != 0:
        devops_toolset.core.log_tools.log_list(log_before_err, devops_toolset.core.log_tools.LogLevel.error)
        devops_toolset.core.log_tools.log_stdouterr(err, devops_toolset.core.log_tools.LogLevel.error)
        devops_toolset.core.log_tools.log_list(log_after_err, devops_toolset.core.log_tools.LogLevel.error)


if __name__ == "__main__":
    help(__name__)



"""Parses files for getting information from them"""

import json
import logging
import pathlib
import re
import xml.etree.ElementTree as ElementTree
import devops_toolset.project_types.wordpress.constants as wp_constants
from typing import List

from devops_toolset.core.app import App
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.filesystem.Literals import Literals as FileSystemLiterals
from devops_toolset.project_types.wordpress.Literals import Literals as WordpressLiterals


app: App = App()
platform_specific = app.load_platform_specific("environment")
literals = LiteralsCore([FileSystemLiterals])
wp_literals = LiteralsCore([WordpressLiterals])


def get_xml_file_entity_text(entity_xpath: str, xml_file_path: str) -> dict:
    """Gets the XML entity tag and value that matches the XPath expression.

    Supported XPath syntax is documented here:
    https://docs.python.org/3/library/xml.etree.elementtree.html#supported-xpath-syntax

    Args:
        entity_xpath: Path to the node or attribute to be matched.
        xml_file_path: Path to the XML file.

    Returns:
        Dict with tag and value.
    """
    xml_tree = ElementTree.parse(xml_file_path)
    entity = xml_tree.find(entity_xpath)
    return {entity.tag: entity.text}


def parse_project_xml_data(add_environment_variables: bool = True, project_xml_path: str = None) -> dict:
    """Reads the /project.xml file and returns a dict with its data.

    XML elements are upper cased, underscored and prepended with parent name.

    Args:
        add_environment_variables: If True it adds every element of the dict as
            an environment variable.
        project_xml_path: Path to the project.xml file.
    """

    if project_xml_path is None:
        project_xml_path = pathlib.Path.joinpath(app.settings.project_xml_path, "project.xml")
    else:
        project_xml_path = pathlib.Path(project_xml_path)
    logging.debug(literals.get("fs_project_path_is").format(path=project_xml_path))
    xml = ElementTree.parse(str(project_xml_path)).getroot()

    environment_variables = {}
    for e in xml:
        environment_variables[f"{xml.tag}_{e.tag}".upper()] = e.text

    if add_environment_variables:
        platform_specific.create_environment_variables(environment_variables)

    return environment_variables


def parse_theme_metadata(css_file_content: bytes, tokens: List[str], add_environment_variables: bool = False) -> dict:
    """ Reads a style.css data and returns its metadata as a dict based on the tokens list.
    Args:
        css_file_content: Data content of the style.css file used to retrieve metadata from.
        tokens: List of strings representing parts of the metadata to be retrieved
        add_environment_variables: If True it adds every element of the dict as an environment variable.

    Returns: Metadata content in a dict

    """
    result = dict()
    environment_variables = dict()
    logging.info(wp_literals.get("wp_parsing_theme_metadata"))

    # Convert css binary data to text
    css_str_content = css_file_content.decode("utf-8")

    # For each token in tokens
    for token in tokens:

        # Build the regex to retrieve data
        regex = token + wp_constants.theme_metadata_parse_regex
        logging.debug(wp_literals.get("wp_parsing_theme_regex").format(regex=regex))

        # Search with regex
        matches = re.search(regex, css_str_content)

        if matches is not None and matches.group(1):
            match = matches.group(1)
            logging.info(wp_literals.get("wp_parsing_theme_matches_found").format(
                token=token, content=match))
            result[token] = match

            # If add_environment_variables, then prepare match as an environment variables
            if add_environment_variables:
                environment_variables[f"theme_{token}".upper()] = match

        else:
            logging.warning(wp_literals.get("wp_parsing_theme_no_matches_found").format(token=token))

    # When environment variables present, then create them
    if len(environment_variables) > 0:
        platform_specific.create_environment_variables(environment_variables)

    return result


def parse_json_file(path: str) -> dict:
    """Parses a JSON file as a dict.

    Args:
        path: Path to the JSON file.

    Returns:
        Dictionary with the file contents.
    """

    with open(path, "r") as json_file:
        return json.load(json_file)


if __name__ == "__main__":
    help(__name__)



""" Helper functions for related Git-Flow tasks automation.
See https://docs.google.com/drawings/d/1orJmg2Eh7iAN5oHvStpyHjeMVn9Zq4wcAE4IXJg_huE/edit """

# ! python
import re
from devops_toolset.tools.constants import dev_branch_regex, release_branch_regex, main_branch_regex


def is_branch_suitable_for_tagging(branch: str) -> bool:
    """ Returns True when branch is suitable for tagging, False if it's not

    Args:
        branch: Branch name to check
    """
    return (
            re.search(dev_branch_regex, branch) is not None
            or re.search(release_branch_regex, branch) is not None
            or re.search(main_branch_regex, branch) is not None
    )


if __name__ == "__main__":
    help(__name__)



"""This file contains common code for all the platforms"""

from devops_toolset.core.app import App
from devops_toolset.core.CommandsCore import CommandsCore
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.devops_platforms.commands import Commands as CommonCommands
from devops_toolset.devops_platforms.Literals import Literals as CommonLiterals
from devops_toolset.devops_platforms.azuredevops.Literals import Literals as AzureDevOpsLiterals
import devops_toolset.tools.cli as cli
import logging

app: App = App()
commands = CommandsCore([CommonCommands])
literals = LiteralsCore([CommonLiterals, AzureDevOpsLiterals])


def echo_environment_variable(environment_variable_name: str) -> str:
    """Echoes an environment variable. This function is common to all and
    limited to operating systems that support the echo command.

    Args:
        environment_variable_name: Name of the environment variable to echo.

    Returns:
        The environment variable value.
    """
    return cli.call_subprocess_with_result(commands.get("echo").format(
        variable=environment_variable_name
    ))


def log_environment_variables(platform_keys: dict):
    """Logs all environment variables for this platform and process.

    Args:
        platform_keys: dict with key-value pairs of platform variables.
    """

    spaces: int = len(max(platform_keys, key=len)) + 5

    for key, value in platform_keys.items():
        logging.info(literals.get("environment_variable_log").format(
            key=str(key).ljust(spaces, "."),
            value=value
        ))


if __name__ == "__main__":
    help(__name__)



"""Environment-related functionality for Aws"""

from enum import Enum
from devops_toolset.core.app import App
from devops_toolset.core.CommandsCore import CommandsCore
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.devops_platforms.Literals import Literals as CommonLiterals
from devops_toolset.devops_platforms.aws.Literals import Literals as AwsLiterals
from devops_toolset.project_types.linux.commands import Commands as LinuxCommands
import devops_toolset.devops_platforms.common
import logging
import os

app: App = App()
literals = LiteralsCore([CommonLiterals, AwsLiterals])
linux_commands = CommandsCore([LinuxCommands])


class ResultType(Enum):
    """Result types for a task"""
    success = "Succeeded"
    partial_success = "SucceededWithIssues"
    fail = "Failed"


def create_environment_variables(key_value_pairs: dict):
    """Creates environment variables

    Args:
        key_value_pairs: Key-value pair dictionary
    """

    for key, value in key_value_pairs.items():
        logging.info(literals.get("platform_created_ev").format(key=key, value=value))
        os.environ[key] = value


def end_task(result_type: ResultType):
    """Ends the current task

    Args:
        result_type: Result type of the task
    """

    if result_type == ResultType.fail:
        raise EnvironmentError()


def get_platform_variable_dict() -> dict:
    """Gets all keys for the environment variables defined by default in the
    platform. Values are replaced automatically by AWS before the script is
    executed.

    More info at:
        https://docs.aws.amazon.com/codepipeline/latest/userguide/reference-variables.html#reference-variables-list

    Returns
        List with all variables
    """

    return {
        "BuildVariables.EnvVar": "#{BuildVariables.EnvVar}",
        "codepipeline.PipelineExecutionId": "#{codepipeline.PipelineExecutionId}",
        "DeployVariables.OperationId": "#{DeployVariables.OperationId}",
        "DeployVariables.StackName": "#{DeployVariables.StackName}",
        "DeployVariables.StackSetId": "#{DeployVariables.StackSetId}",
        "SourceVariables.AuthorDate": "#{SourceVariables.AuthorDate}",
        "SourceVariables.BranchName": "#{SourceVariables.BranchName}",
        "SourceVariables.CommitId": "#{SourceVariables.CommitId}",
        "SourceVariables.CommitMessage": "#{SourceVariables.CommitMessage}",
        "SourceVariables.CommitterDate": "#{SourceVariables.CommitterDate}",
        "SourceVariables.CommitUrl": "#{SourceVariables.CommitUrl}",
        "SourceVariables.ConnectionArn": "#{SourceVariables.ConnectionArn}",
        "SourceVariables.ETag": "#{SourceVariables.ETag}",
        "SourceVariables.FullRepositoryName": "#{SourceVariables.FullRepositoryName}",
        "SourceVariables.ImageDigest": "#{SourceVariables.ImageDigest}",
        "SourceVariables.ImageTag": "#{SourceVariables.ImageTag}",
        "SourceVariables.ImageURI": "#{SourceVariables.ImageURI}",
        "SourceVariables.RegistryId": "#{SourceVariables.RegistryId}",
        "SourceVariables.RepositoryName": "#{SourceVariables.RepositoryName}",
        "SourceVariables.VersionId": "#{SourceVariables.VersionId}",
        "TestVariables.testRunId": "#{TestVariables.testRunId}",
    }


def log_environment_variables(platform_keys: dict):
    """Logs all environment variables for this platform and process.

    Args:
        platform_keys: List of platform variables.
    """

    devops_toolset.devops_platforms.common.log_environment_variables(platform_keys)


if __name__ == "__main__":
    help(__name__)



"""node module commands"""

from devops_toolset.core.ValueDictsBase import ValueDictsBase
from devops_toolset.core.app import App

app: App = App()


class Commands(ValueDictsBase):
    """Commands for the node module."""

    # Add your node literal dictionaries here
    _commands = {
        "npm_install": "npm install {folder}",
        "npm_run": "npm run {command} {silent} {if_present} {extra_args}"
    }



"""Contains tools for logging"""

import logging
from enum import Enum
from typing import List


class LogLevel(Enum):
    """https://docs.python.org/3/library/logging.html#levels"""
    critical = 50,
    error = 40,
    warning = 30,
    info = 20,
    debug = 10,


def log_list(logs_list: List[str], level: LogLevel):
    """Logs a list using the specified level.

    Args:
        logs_list: List of strings to be logged.
        level: Logging level.
    """
    if logs_list:
        for log in logs_list:
            logging.log(level.value[0], log)


def log_stdouterr(output: bytes, level: LogLevel):
    """Logs stdout or stderr outputs.

    It splits lines using the universal line ending and decodes content to
    UTF-8.

    Args:
        output: stdout or stderr output to be logged.
        level: Logging level.
    """

    for line in output.splitlines():
        logging.log(level.value[0], line.decode(encoding="utf-8", errors="ignore"))


def log_indented_list(header: str, list_values: List[str], level: LogLevel):
    """Logs an indented list with a not indented header.

    Args:
        header: String being logged first, not indented.
        list_values: List of values to be logged, intented.
        level: Logging level.
    """

    if list_values:

        logging.log(level.value[0], header)

        for value in list_values:
            logging.log(level.value[0], f"\t{value}")


def get_parameter_value_list(local_values: dict) -> List:
    """Creates a list with all parameter values.

    Args:
        local_values: locals from the function.

    Returns:
        List with argument name and value pairs.
    """

    argument_list = []

    for key, value in local_values.items():
        argument_list.append(f"{key} = {value}")

    return argument_list



""" This script will hold general purpose utils for a remote linux host """

from devops_toolset.core.app import App
from devops_toolset.core.CommandsCore import CommandsCore
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.project_types.linux.commands import Commands as LinuxCommands
from devops_toolset.project_types.linux.Literals import Literals as LinuxLiterals
import logging
import pathlib
import devops_toolset.tools.cli as tools_cli

app: App = App()
literals = LiteralsCore([LinuxLiterals])
commands = CommandsCore([LinuxCommands])


def edit_in_place(search_for: str, replace_with: str, file_path: str):
    """ Performs an edit-in-place operation by replacing a value on a file using sed -i command
    Arguments:
        search_for: The value will be replaced.
        replace_with: The value "search_for" will be replaced for.
        file_path: Path of the destination file to perform the edit operation.

    Raises: FileNotFoundError -> When file_path does not exist
    """

    if not pathlib.Path(file_path).exists():
        message = literals.get("file_not_exist_err").format(path=file_path)
        logging.error(message)
        raise FileNotFoundError()

    tools_cli.call_subprocess(commands.get("edit_in_place").format(
        search_for=search_for,
        replace_with=replace_with,
        file_path=file_path
    ), log_before_out=[literals.get("edit_in_place_post").format(
        search_for=search_for, replace_with=replace_with, path=file_path)],
     log_after_err=[literals.get("edit_in_place_err").format(
         search_for=search_for, replace_with=replace_with, path=file_path)])


def edit_multiple_in_place(replacements: dict, file_path: str):
    """ Performs an edit-in-place operation by replacing a value on a file using sed -i command
    Arguments:
        replacements: Dict containing: key -> string to replace -- value -> replacement
        file_path: Path of the destination file to perform the edit operation.
    """

    for key, value in replacements.items():
        edit_in_place(key, value, file_path)


if __name__ == "__main__":
    help(__name__)



"""Defines constants for the module."""

from enum import Enum


class Directions(Enum):
    """Defines sorting and finding directions"""

    ASCENDING = 1,
    DESCENDING = 2


class FileNames(object):
    """Defines constants for file names.

    Attributes:
        GITIGNORE_FILE: .gitignore file.
        PROJECT_FILE: Project definition file.
        TEMP_DIRECTORY: Temporary directory name.
    """

    GITIGNORE_FILE = ".gitignore"
    PROJECT_FILE = "project.xml"
    TEMP_DIRECTORY = "__temp"


class FileType(Enum):
    """Defines the file type of a file to be written.

    To be added to w or b options when using builtins.open().

    e.g:
        To read a text file:
            with.open(file, f"r{FileType.TEXT}")... => r
        To write a binary file:
            with.open(file, f"w{FileType.BINARY}")... => wb

    Attributes:
        TEXT: Text file suffix
        BINARY: Binary file suffix
    """

    TEXT = ""
    BINARY = "b"



"""tools module literals."""

from devops_toolset.core.ValueDictsBase import ValueDictsBase
from devops_toolset.core.app import App

app: App = App()


class Literals(ValueDictsBase):
    """ValueDicts for the tools module."""

    # Add your core literal dictionaries here
    _info = {
        "cli_return_code": _("Process terminated with return code {code}"),
        "git_purging_gitkeep": _("Purging .gitkeep file at {path}"),
        "git_repo_to_be_created": _("The repository is going to be created"),
        "git_repo_created": _("The repository has been created"),
        "val_path_argument_not_valid": _("The path specified in the {argument} argument is invalid or does not exist."),
        "git_init_repo": _("Do you want me to initialize a local Git repository for you?"),
        "git_before_adding_project_structure_files_to_stage": _("Adding files to the stage"),
        "git_after_adding_project_structure_files_to_stage": _("Added files to the stage"),
        "git_add_project_structure_message": _("File project structure created"),
        "git_before_project_structure_commit": _("Creating file project structure commit"),
        "git_after_project_structure_commit": _("Committed project structure files"),
        "git_before_change_branch": _("Changing branch to master"),
        "git_after_change_branch": _("Changed branch to master successfully"),
        "git_push_tag_init": _("Start git tagging push to origin -> tag_name: {tag_name}"),
        "git_push_tag_delete_init": _("Start git tag deletion push to origin -> tag_name: {tag_name}"),
        "git_tag_add_init": _("Start git tagging: tag {tag_name} on commit {commit_name}"),
        "git_tag_delete_init": _("Start git tag delete: {tag_name}"),
        "http_response": _("HTTP response from {url} => {response}"),
    }

    _errors = {
        "git_err_create_repo": _("Git error: repository couldn't be created"),
        "git_non_valid_dir_path": _("Path must be an existent dir."),
        "git_regex1cg": _("RegEx must have 1 capture group. No less, no more."),
        "git_err_commit_project_structure": _("Git error: project structure files couldn't be committed"),
        "git_err_adding_project_structure_files_to_stage": _("Files couldn't be staged"),
        "git_err_changing_branch": _("Changing to master branch couldn't be done"),
        "git_tag_add_err": _("There was an error while tagging commit {commit_name}"),
        "git_tag_delete_err": _("There was an error while deleting tag {tag_name}"),
        "git_push_tag_err": _("There was an error while pushing tag {tag_name} to origin"),
        "git_push_tag_delete_err": _("There was an error while pushing tag {tag_name} deletion to origin"),
    }



"""Helper functions fot HTTP-related tasks."""

from devops_toolset.core.app import App
from devops_toolset.core.CommandsCore import CommandsCore
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.tools.commands import Commands as ToolsCommands
from devops_toolset.tools.Literals import Literals as ToolsLiterals

import logging
import re
import requests

app: App = App()
platform_specific = app.load_platform_specific("environment")
literals = LiteralsCore([ToolsLiterals])
commands = CommandsCore([ToolsCommands])


def get_public_ip_address(
        public_service_url: str = commands.get("public_ip_address_service_url"),
        environment_variable_name: str = "DT_PUBLIC_IP_ADDRESS") -> [str, None]:
    """Gets the machine public IP address using an external service.

    Args:
        public_service_url: URL of the public service to be requested. Defaults
            to checkip.dyndns.com
        environment_variable_name: Name of the environment variable to be
            created. Defaults to "DT_PUBLIC_IP_ADDRESS".

    Returns:
        Public IP address of the endpoint that calls this function.
    """

    regex = r"((?:\d{1,3}\.?){4})"

    response = requests.get(public_service_url)
    logging.info(literals.get("http_response").format(
        url=public_service_url,
        response=response.content.decode("utf8")
    ))

    match = re.search(regex, response.content.decode("utf-8"))
    if match is not None:
        ip_address = match.groups()[0]
        platform_specific.create_environment_variables({environment_variable_name: ip_address})
        return ip_address
    else:
        return None


if __name__ == "__main__":
    help(__name__)



"""This script downloads devops-toolset from the GitHub repository (not from
    the feed).

    NOTE:
        This script should only contain pure Python code without any reference
            to the toolset, since it will be called independently."""

import argparse
import logging
import os
import pathlib
import requests
import shutil
import zipfile

logging.basicConfig(level=logging.INFO)
logging.Formatter("%(asctime)s %(levelname)-8s %(module)-15s %(message)s")
logger = logging.getLogger(__name__)


def main(destination_path: str, branch: str):
    """Downloads devops-toolset from the GitHub repository

    Args:
        destination_path: Path where devops-toolset will be downloaded and decompressed.
        branch: What branch to download from the repository.
    """

    toolset_name = "devops-toolset"
    dashed_branch = branch.replace("/", "-")
    internal_directory = f"{toolset_name}-{dashed_branch}"

    # Download the toolset
    destination_path_object, full_destination_path = download_toolset(branch, destination_path, toolset_name)

    # Decompress the toolset
    internal_directory_full_path, temporary_extraction_path = \
        decompress_toolset(destination_path_object, full_destination_path, internal_directory)

    # Delete the temporary files
    cleanup(full_destination_path, internal_directory_full_path, temporary_extraction_path)

    # Delete myself
    os.remove(__file__)


def cleanup(full_destination_path, internal_directory_full_path, temporary_extraction_path):
    """Deletes temporary files

    Args:
        full_destination_path: Path where devops-toolset will be downloaded and decompressed joined to the toolset-name
        internal_directory_full_path: Path where devops-toolset will be extracted joined to
        {toolset_name}-{dashed_branch}
        temporary_extraction_path: Path devops-toolset will be extracted

        """
    os.rmdir(internal_directory_full_path)
    logger.info(f"Deleted directory {internal_directory_full_path}")
    os.rmdir(temporary_extraction_path)
    logger.info(f"Deleted directory {temporary_extraction_path}")
    os.remove(full_destination_path)
    logger.info(f"Deleted file {full_destination_path}")


def decompress_toolset(destination_path_object, full_destination_path, internal_directory):
    """Decompresses devops-toolset

    Args:
        destination_path_object: Path where devops-toolset will be downloaded and decompressed.
        full_destination_path: Path where devops-toolset will be downloaded and decompressed joined to the toolset-name
        internal_directory: {toolset_name}-{dashed_branch}

        """
    temporary_extraction_path = pathlib.Path.joinpath(destination_path_object, "__temp")
    with zipfile.ZipFile(full_destination_path, "r") as zip_file:
        zip_file.extractall(temporary_extraction_path)
    internal_directory_full_path = pathlib.Path.joinpath(temporary_extraction_path, internal_directory)
    items = os.listdir(internal_directory_full_path)
    for item in items:
        from_path = pathlib.Path.joinpath(internal_directory_full_path, item)
        to_path = pathlib.Path.joinpath(destination_path_object, item)
        shutil.move(from_path, to_path)
    logger.info(f"devops-toolset decompressed to {destination_path_object}")
    return internal_directory_full_path, temporary_extraction_path


def download_toolset(branch, destination_path, toolset_name):
    """Downloads devops-toolset from the GitHub repository

        Args:
            toolset_name: The name of the zip downloaded.
            destination_path: Path where devops-toolset will be downloaded and
                decompressed.
            branch: What branch to download from the repository.
        """
    destination_path_object = pathlib.Path(destination_path)
    full_destination_path = pathlib.Path.joinpath(destination_path_object, f"{toolset_name}.zip")
    response = requests.get(f"https://github.com/aheadlabs/devops-toolset/archive/{branch}.zip")
    if not os.path.exists(destination_path):
        os.mkdir(destination_path)
    with open(full_destination_path, "wb") as zip_file:
        zip_file.write(response.content)
    logger.info(f"devops-toolset downloaded to {full_destination_path}")
    return destination_path_object, full_destination_path


def is_valid_path(path: str) -> bool:
    """Checks if a path is valid"""

    if path is None or path.strip() == "":
        return False

    return True


class PathValidator(argparse.Action):
    """Validates a path"""
    def __init__(self, option_strings, dest, **kwargs):
        super(PathValidator, self).__init__(option_strings, dest.replace("-", "_"), **kwargs)

    def __call__(self, parent_parser, namespace, values, option_string=None):
        setattr(namespace, self.dest, values)

        if not is_valid_path(values):
            raise ValueError("The argument {argument} is not valid.".format(argument=self.dest))


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("destination-path", action=PathValidator)
    parser.add_argument("--branch", default="master")
    args, args_unknown = parser.parse_known_args()

    main(args.destination_path, args.branch)



"""Supports al compression / decompression operations in the file system."""

from devops_toolset.core.app import App
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.filesystem.Literals import Literals as FileSystemLiterals
import devops_toolset.filesystem.constants as constants
import devops_toolset.filesystem.paths
import logging
import os
import pathlib
import shutil
import zipfile

app: App = App()
literals = LiteralsCore([FileSystemLiterals])


def download_and_unzip_file(url: str, destination: str, delete_after_unzip: bool = True, unzip_root: str = None):
    """Downloads and unzips a file from a URL.

    Args:
        url: Where to download the file from.
        destination: Where to download and unzip the file.
        delete_after_unzip: If True it deletes the file after unzipping it.
        unzip_root: Determines which directory inside the zip file is the root
            directory.
    """

    file_name, file_path = devops_toolset.filesystem.paths.download_file(url, destination, constants.FileType.BINARY)
    destination_path = pathlib.Path(destination)
    temp_extraction_path = pathlib.Path.joinpath(destination_path, constants.FileNames.TEMP_DIRECTORY)

    with zipfile.ZipFile(file_path, "r") as zip_file:
        zip_file.extractall(temp_extraction_path)

    if unzip_root is not None:
        temp_rooted_extraction_path = pathlib.Path.joinpath(temp_extraction_path, unzip_root)
        for path, directories, files in os.walk(temp_rooted_extraction_path):
            path_object = pathlib.Path(path)
            for file in files:
                shutil.move(str(pathlib.Path.joinpath(path_object, file)), destination)
            for directory in directories:
                shutil.move(str(pathlib.Path.joinpath(path_object, directory)), destination)
        os.rmdir(temp_rooted_extraction_path)
        os.rmdir(temp_extraction_path)

    if delete_after_unzip:
        os.remove(file_path)


def zip_directory(directory_path: str, file_path, internal_path_prefix: str = ""):
    """Creates a ZIP file of the contents of the specified directory path.

    Args:
        directory_path: Path to the directory to be zipped (directory will not
            be included in the ZIP file).
        file_path: Path to the file to be created.
        internal_path_prefix: Prefix to be added to all the internal paths.
            Must end with /

    Returns:
        Path to the created ZIP file.
    """
    with zipfile.ZipFile(file_path, "w") as output_file:
        for directory, subfolders, files in os.walk(directory_path):
            for file in files:
                current_path = pathlib.Path.joinpath(pathlib.Path(directory), file)
                zip_internal_basepath = pathlib.Path(directory.replace(directory_path, "")).as_posix()
                zip_internal_path = f"{internal_path_prefix}{zip_internal_basepath}/{file}"
                output_file.write(current_path, zip_internal_path)
                logging.debug(literals.get("fs_zip_added_file").format(
                    zip_file_name=os.path.basename(file_path),
                    added_file=zip_internal_path
                ))


def read_text_file_in_zip(zip_file_path: str, text_file_path: str) -> bytes:
    """Reads a text file that is enclosed inside a ZIP file.

    Args:
        zip_file_path: Path to the ZIP file.
        text_file_path: Path to the text file inside the ZIP file.

    Returns:
        Content of the text file
    """
    with zipfile.ZipFile(zip_file_path, "r") as zip_file:
        return zip_file.read(text_file_path)


if __name__ == "__main__":
    help(__name__)



"""Angular module commands"""

from devops_toolset.core.ValueDictsBase import ValueDictsBase
from devops_toolset.core.app import App

app: App = App()


class Commands(ValueDictsBase):
    """ Commands for the Angular module."""

    # Add your Angular commands dictionaries here
    _commands = {}



"""Gets the AWS resources in your account (uses your default machine
authentication settings)"""

import argparse
import json
import devops_toolset.tools.argument_validators
import devops_toolset.tools.cli as cli


def main(json_path: str, hosted_zone: str):
    """Gets the AWS resources and writes them to a JSON file.

    Args:
        json_path: Path to the JSON file to be written.
        hosted_zone: DNS zone ID to be queried.
            ie: /hostedzone/ABCDEFGHIJKL123456789 (must pass everything)
    """

    resources: dict = {}

    resources["commit_repositories"] = json.loads(cli.call_subprocess_with_result(
        "aws codecommit list-repositories --query \"repositories[].repositoryName\""))
    resources["codebuild_projects"] = json.loads(cli.call_subprocess_with_result(
        "aws codebuild list-projects --query \"projects[]\""))
    resources["codepipeline_pipelines"] = json.loads(cli.call_subprocess_with_result(
        "aws codepipeline list-pipelines --query \"pipelines[].name\""))
    resources["codeartifact_repositories"] = json.loads(cli.call_subprocess_with_result(
        "aws codeartifact list-repositories"))
    resources["event_buses"] = json.loads(cli.call_subprocess_with_result(
        "aws events list-event-buses --query \"EventBuses[].Name\""))
    resources["event_rules"] = json.loads(cli.call_subprocess_with_result(
        "aws events list-rules"))
    resources["ec2_instances"] = json.loads(cli.call_subprocess_with_result(
        "aws ec2 describe-instances"))
    resources["elasticbeanstalk_applications"] = json.loads(cli.call_subprocess_with_result(
        "aws elasticbeanstalk describe-applications"))
    resources["elasticbeanstalk_environments"] = json.loads(cli.call_subprocess_with_result(
        "aws elasticbeanstalk describe-environments"))
    resources["s3_buckets"] = json.loads(cli.call_subprocess_with_result(
        "aws s3api list-buckets --query \"Buckets[].Name\""))
    resources["route53_hosted_zones"] = json.loads(cli.call_subprocess_with_result(
        "aws route53 list-hosted-zones --query \"HostedZones[].Name\""))

    if hosted_zone is not None:
        resources["route53_recordsets"] = json.loads(cli.call_subprocess_with_result(
            f"aws route53 list-resource-record-sets --hosted-zone-id {hosted_zone}"))

    resources["iam_custom_policies"] = json.loads(cli.call_subprocess_with_result(
        "aws iam list-policies --scope Local --query \"Policies[].PolicyName\""))
    resources["iam_roles"] = json.loads(cli.call_subprocess_with_result(
        "aws iam list-roles --query \"Roles[].RoleName\""))

    with open(json_path, 'w') as data_file:
        json.dump(resources, data_file)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("json-path", action=tools.argument_validators.PathValidator)
    parser.add_argument("--hosted-zone", default=None)
    args, args_unknown = parser.parse_known_args()

    main(args.json_path, args.hosted_zone)



"""devops_toolset.devops_platforms.azuredevops module literals."""

from devops_toolset.core.ValueDictsBase import ValueDictsBase
from devops_toolset.core.app import App

app: App = App()


class Literals(ValueDictsBase):
    """ValueDicts for the devops_toolset.devops_platforms.azuredevops module."""

    # Add your core literal dictionaries here
    _info = {
        "azdevops_command": _("Command: {command}"),
        "azdevops_download_package_manually": _("Please, download the packages from the Azure DevOps feed manually."),
        "platform_created_ev": _("Created environment variable {key} with value {value}"),
    }
    _errors = {
        "azdevops_status_code": _("Status code for the request: {status_code}"),
        "azdevops_token_not_found": _("Azure DevOps token was not found in the **kwargs parameter."),
    }



"""dotnet module literals"""

from devops_toolset.core.app import App
from devops_toolset.core.ValueDictsBase import ValueDictsBase

app: App = App()


class Literals(ValueDictsBase):
    """ValueDicts for the dotnet module."""

    _titles = {
        "dotnet_ci_title_pipeline_kickoff": _(".NET CI pipeline kick off"),
    }
    _info = {
        "dotnet_build_before": "Launching dotnet build inside {path}. Please wait..",
        "dotnet_ef_database_drop": "Dropping the database...",
        "dotnet_ef_database_reset": "Reverting all migrations...",
        "dotnet_ef_first_migration_not_applied": "First migration not applied: {migration_name} ({migration_date})",
        "dotnet_ef_got_environments": "I got these environments: {environments}.",
        "dotnet_ef_migrations_info":
            "Number of migrations: {number}, applied migrations: {applied}, last applied migration: {name}",
        "dotnet_ef_migrations_list": "Listing migrations (will take a while)...",
        "dotnet_ef_migrations_list_output": "I got this output getting the migration's list:\n{output}",
        "dotnet_ef_migrations_script": "Generating SQL script (will take a while)...",
        "dotnet_ef_no_pending_migrations": "There are no pending migrations to be applied.",
        "dotnet_ef_script_being_generated": "SQL migration script being generated: {script_path}",
        "dotnet_ef_script_executing_command": "Executing command => {command}",
        "dotnet_ef_script_for_environment": "Generating SQL migration script for {environment} environment...",
        "dotnet_ef_utils_appsettings_dev_environment_skipped":
            "Skipped development environment as include_development is False",
        "dotnet_ef_utils_appsettings_files_matched": "I got these appsettings files matched: {files}",
        "dotnet_ef_utils_appsettings_environment_matched": "Environment {environment} matched on: {filename}",
        "dotnet_ef_utils_appsettings_no_environment_matched": "No environment matched on: {filename}",
        "dotnet_ef_utils_getting_appsettings_files": "Getting appsettings files from: {path}",
        "dotnet_git_tag": "Adding tag {tag_name} to the commit {commit_name} in branch {branch_name}",
        "dotnet_project_version": "The project version is {version}",
        "dotnet_restore_before": "Launching dotnet restore inside {path}. Please wait..",
    }
    _errors = {
        "dotnet_restore_err": "Something went wrong while restoring {path}. Please check the logs and try again.",
        "dotnet_build_err": "Something went wrong while building {path}. Please check the logs and try again.",
    }



"""Rollbacks a database using a dump, removing first all existing tables."""

import argparse
from devops_toolset.core.app import App
from devops_toolset.core.CommandsCore import CommandsCore
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.tools.commands import Commands as ToolsCommands
from devops_toolset.tools.Literals import Literals as ToolsLiterals
from devops_toolset.project_types.wordpress.Literals import Literals as WordpressLiterals
from devops_toolset.project_types.wordpress import wp_cli

app: App = App()
literals = LiteralsCore([WordpressLiterals, ToolsLiterals])
commands = CommandsCore([ToolsCommands])


def main(wordpress_path: str, database_dump_path: str, quiet: bool):
    """Rollbacks a database using a dump, dropping and re-creating the database

    Args:
        wordpress_path: Path to WordPress directory.
        database_dump_path: Path to the database dump file to be restored.
        quiet: If True, no questions are asked and defaults are assumed.
    """

    # Drop the database and create an empty one
    wp_cli.reset_database(wordpress_path, quiet)

    # Imports a dump in the newly created database
    wp_cli.import_database(wordpress_path, database_dump_path)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("wordpress-path", action=tools.argument_validators.PathValidator)
    parser.add_argument("database-dump-path", action=tools.argument_validators.PathValidator)
    parser.add_argument("--quiet", action="store_true", default=False)
    args, args_unknown = parser.parse_known_args()

    tools.cli.print_title(literals.get("wp_title_wordpress_rollback_db"))
    main(args.wordpress_path, args.database_dump_path, args.quiet)



"""Tools for editing files"""

import json
import logging
import os
import pathlib
import xml.etree.ElementTree as ElementTree

from devops_toolset.core.app import App
from devops_toolset.core.LiteralsCore import LiteralsCore
from devops_toolset.filesystem.Literals import Literals as FileSystemLiterals
from devops_toolset.project_types.wordpress.Literals import Literals as WordpressLiterals


app: App = App()
platform_specific = app.load_platform_specific("environment")
literals = LiteralsCore([FileSystemLiterals])
wp_literals = LiteralsCore([WordpressLiterals])


def is_file_empty(path: str) -> bool:
    """Checks if a file is empty.

    Args:
        path: Path to the file to be checked.

    Returns:
        True if file is empty.
    """

    return os.path.getsize(path) == 0


def strip_utf8_bom_character_from_file(path: str):
    """Removes byte-order-mark character from the beginning of the UTF8 file"""

    logging.info(literals.get("fs_strip_bom").format(path=path))

    with open(path, "r", encoding="utf-8-sig") as file:
        content = file.read()
        logging.info(literals.get("fs_file_content").format(content=content))

        with open(path, "w", encoding="utf-8") as file_w:
            logging.info(literals.get("fs_writing_file").format(path=path))
            file_w.write(content)


def update_json_file_key_text(key_path: list, key_value: str, json_file_path: str):
    """Updates a JSON file with the given value.

    Args:
        key_path: Path to the key to be updated. Each element of the list is a
            level in the nested key structure.
        key_value: CValue to be set.
        json_file_path: Path to the JSON file.
    """

    if len(key_path) == 0:
        raise ValueError(literals.get("list_length_zero"))

    if len(key_path) > 3:
        raise ValueError(literals.get("list_length_higher_than").format(length=3))

    content: dict = {}
    json_file_path_obj: pathlib.Path = pathlib.Path(json_file_path)

    with open(json_file_path_obj, "r") as json_file:
        content = json.load(json_file)

    if len(key_path) == 1:
        content[key_path[0]] = key_value

    if len(key_path) == 2:
        content[key_path[0]][key_path[1]] = key_value

    if len(key_path) == 3:
        content[key_path[0]][key_path[1]][key_path[2]] = key_value

    with open(json_file_path_obj, "w") as json_file:
        json.dump(content, json_file)

    # TODO(ivan.sainz) Refactor this code using reduce()


def update_xml_file_entity_text(entity_xpath: str, entity_value: str, xml_file_path: str):
    """Updates an XML file with the given value.

    Supported XPath syntax is documented here:
    https://docs.python.org/3/library/xml.etree.elementtree.html#supported-xpath-syntax

    Args:
        entity_xpath: Path to the node or attribute to be updated.
        entity_value: Value to be set.
        xml_file_path: Path to the XML file.
    """

    xml_tree = ElementTree.parse(xml_file_path)
    entity = xml_tree.find(entity_xpath)
    entity.text = entity_value
    xml_tree.write(xml_file_path)


if __name__ == "__main__":
    help(__name__)
